--[[
	Define all Global Variables here
]]

-- Global Variables that are not supposed to be on-the-fly user configurable.
perl_target_locked = 0;
perl_loaded		= 0;
perl_locked		= 1;
perl_init		= 1;  -- This variable is used to tell all the initial function that they can now run.

function Perl_ShowMessage(cMsg)
	local str = "|c00FF7F00"..event.."|r";
	if (arg1) then str = str..", "..arg1; end
	if (arg2) then str = str..", "..arg2; end
	if (arg3) then str = str..", "..arg3; end
	if (arg4) then str = str..", "..arg4; end
	if (arg5) then str = str..", "..arg5; end
	if (arg6) then str = str..", "..arg6; end
	if (arg7) then str = str..", "..arg7; end
	if (arg8) then str = str..", "..arg8; end
	if (arg9) then str = str..", "..arg9; end

	if (cMsg) then
		str = cMsg.." - "..str;
	end

	ChatFrame7:AddMessage("|c00007F7F"..this:GetName().."|r - "..str);
end

function Perl_Globals_OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
end

function Perl_Globals_OnEvent(event)
	-- Between sessions variable saving.
	this:UnregisterEvent(event);

	if ((not Perl_Config) or (not Perl_Config.BarTextures)) then
		-- Create the global variable
		Perl_Config = {};

		-- Set the "Config" Version (If you use the program version it makes things easier as you only need to modify it in the localization file)
		Perl_Config.ConfigVersion	= Perl_VersionNumber;

		-- Define the defaults for the global variable
		Perl_Config.BarTextures			= 1;	--change to 0 to not show the bar textures
		Perl_Config.Transparency		= 1;	--change to the desired overall transparency, between 0 and 1
		Perl_Config.TextTransparency   		= 0.8;	--change to the desired text transparency, between 0 and 1
		Perl_Config.BackColour			= {["r"] = 0, ["g"] = 0, ["b"] = 0};	--change to desired colour
		Perl_Config.BorderColour		= {["r"] = 0.5, ["g"] = 0.5, ["b"] = 0.5};
		Perl_Config.ArcaneBar			= 1;	--change this to 0 to hide the ArcaneBar
		Perl_Config.OldCastBar			= 1;	--change this to 0 to hide the old cast bar
		Perl_Config.CastTime			= 1;

		-- Target Counters
		Perl_Config.TargetCounters		= 1;
		Perl_Config.TargetCountersSelf		= 1;	-- 0=All, 1=Healers
		Perl_Config.AssistsFrame		= 0;
		Perl_Config.TargettingFrame		= 0;
		Perl_Config.Scale_AssistsFrame		= 1;

		-- CastParty Support
		Perl_Config.CastParty			=1;	--change to 0 to turn off CastParty support
		Perl_Config.CastPartyShow		=0;	--change to 1 to show CastParty frames

		-- Options that modify the player's display
		Perl_Config.ShowPlayerPortrait		= 1;	--change this to 0 to automatically hide the player's portrait
		Perl_Config.ShowPlayerPortrait3D	= 1;	--change this to 0 to use 3D portrait
		Perl_Config.ShowPlayerLevel		= 1;	--change this to 0 to hide the player's level
		Perl_Config.ShowPlayerClassIcon		= 1;	--change this to 0 to hide the player's class icon
		Perl_Config.ShowPlayerXPBar		= 0;	--change to 1 to show player XP bar
		Perl_Config.ShowPlayerPVPRank 		= 1;	--change to 0 to hide the target's PVP rank
		Perl_Config.ShowOldPlayerFrame 		= 0;	--change this to 1 to show old blizzard player frame
		Perl_Config.Scale_PlayerFrame  		= 0.8;	--sets the scale of the player frame

		-- Options that modify the target display
		Perl_Config.ShowTargetPortrait		= 1;	--change this to 0 to hide the target's portrait
		Perl_Config.ShowTargetClassIcon		= 1;   	--change this to 0 to hide the target's class icon
		Perl_Config.ShowTargetMobType  		= 1;	--change this to 0 to hide the target's mob type
		Perl_Config.ShowTargetLevel		= 1;	--change this to 0 to hide the target's level
		Perl_Config.ShowTargetElite		= 1;	--change this to 0 to hide display of 'elite' or 'boss'
		Perl_Config.ShowTargetDistance 		= 1;
		Perl_Config.ShowTargetMana		= 1;	--change this to 0 to hide target mana and energy
		Perl_Config.UseCPMeter			= 1;	--change this to 0 to use combo point text display
		Perl_Config.ShowTargetPVPRank  		= 1;	--change to 0 to hide the player's PVP rank
		Perl_Config.Scale_TargetFrame  		= 0.8;	--sets the scale of the target frame

		-- Target of Target options
		Perl_Config.ShowTargetTarget		= 1;
		Perl_Config.Scale_TargetTargetFrame	= 0.8;
		Perl_Config.TargetTargetHistory		= 1;

		-- Options that modify the Party display
		Perl_Config.ShowPartyTarget		= 1;	--change this to 0 to hide party member targets
		Perl_Config.ShowPartyLevel		= 1;	--change this to 0 to hide party levels
		Perl_Config.ShowPartyNames		= 1;	--change this to 0 to hide party names
		Perl_Config.ShowPartyDistance  		= 1;	--change this to 0 to hide distance to party members
		Perl_Config.ShowPartyPercent   		= 1;	--change this to 0 to hide party percentages
		Perl_Config.ShowPartyClassIcon 		= 1;	--change this to 0 to hide party class icons
		Perl_Config.ShowPartyRaid		= 1;	--change to 0 to hide party frames while in a raid group
		Perl_Config.Scale_PartyFrame   		= 0.8;	--sets the scale of the party frame
		Perl_Config.PartyDebuffsBelow  		= 0;
		Perl_Config.ShowPartyNumber		= 1;

		-- Options that modify the pet display
		Perl_Config.PetHappiness       		= 1;	--change this to 0 to hide pet happiness icon
		Perl_Config.ShowPetLevel       		= 1;	--change this to 0 to hide pet level
		Perl_Config.ShowPetXP	       		= 1;
		Perl_Config.Scale_PetFrame     		= 0.8;	--sets the scale of the pet frame
		Perl_Config.ShowPartyPets      		= 1;

		--[[
		Perl_Config.SortRaidByClass    		= 1;
		Perl_Config.ShowRaid	       		= 1;
		Perl_Config.ShowGroup1	       		= 1;
		Perl_Config.ShowGroup2	       		= 1;
		Perl_Config.ShowGroup3	   		= 1;
		Perl_Config.ShowGroup4			= 1;
		Perl_Config.ShowGroup5	       		= 1;
		Perl_Config.ShowGroup6	       		= 1;
		Perl_Config.ShowGroup7	       		= 1;
		Perl_Config.ShowGroup8	       		= 1;
		Perl_Config.ShowGroup9	       		= 1;
		Perl_Config.ShowRaidPercents   		= 1;
		Perl_Config.Scale_Raid	      		=0.8;
		]]

	else
		-- Add new variables here and above (to set defaults).
		-- The reason for this is when you upgrade the variables you need to define the new settings somewhere.
		-- This is important for between version upgrades when the variables change.
		-- There really is no in-game performance loss for running this each version upgrade as it only happens
		-- the first time someone runs the new version, and only once as the first thing it does is update the
		-- version string.

		if ((not Perl_Config.ConfigVersion) or (Perl_Config.ConfigVersion ~= Perl_VersionNumber)) then
			Perl_Config.ConfigVersion = Perl_VersionNumber;	-- Set ConfigVersion to current

			-- This adds the Scale_TargetTargetFrame option if it isn't in the config at the moment,
			-- best to just do this check for each new variable in case somehow the user gets dumped
			-- into this routine.
			if (not Perl_Config.Scale_TargetTargetFrame) then
				Perl_Config.Scale_TargetTargetFrame=0.8;
			end
			if (not Perl_Config.ShowPetXP) then
				Perl_Config.ShowPetXP=1;
			end
		--[[
			if (not Perl_Config.SortRaidByClass) then
				Perl_Config.SortRaidByClass=1;
			end
			if (not Perl_Config.ShowRaid) then
				Perl_Config.ShowRaid= 1;
			end
			if (not Perl_Config.ShowGroup1) then
				Perl_Config.ShowGroup1= 1;
			end
			if (not Perl_Config.ShowGroup2) then
				Perl_Config.ShowGroup2= 1;
			end
			if (not Perl_Config.ShowGroup3) then
				Perl_Config.ShowGroup3= 1;
			end
			if (not Perl_Config.ShowGroup4) then
				Perl_Config.ShowGroup4= 1;
			end
			if (not Perl_Config.ShowGroup5) then
				Perl_Config.ShowGroup5= 1;
			end
			if (not Perl_Config.ShowGroup6) then
				Perl_Config.ShowGroup6= 1;
			end
			if (not Perl_Config.ShowGroup7) then
				Perl_Config.ShowGroup7= 1;
			end
			if (not Perl_Config.ShowGroup8) then
				Perl_Config.ShowGroup8= 1;
			end
			if (not Perl_Config.ShowGroup9) then
				Perl_Config.ShowGroup9= 1;
			end
			if (not Perl_Config.ShowRaidPercents) then
				Perl_Config.ShowRaidPercents= 1;
			end
			if (not Perl_Config.Scale_Raid) then
				Perl_Config.Scale_Raid=0.8;
			end
		]]
			if (not Perl_Config.CastTime) then
				Perl_Config.CastTime=1;
			end
			if (Perl_Config.PartyDebuffsBelow) then
				Perl_Config.PartyDebuffsBelow=0;
			end
			if (not Perl_Config.ShowPartyPets) then
				Perl_Config.ShowPartyPets=1;
			end
		end

		if (Perl_Config.TargetCounters == nil) then
			Perl_Config.TargetCounters = 1;
		end
		if (Perl_Config.TargetCountersSelf == nil) then
			Perl_Config.TargetCountersSelf = 1;
		end
		if (Perl_Config.TargetCountersEnemy == nil) then
			Perl_Config.TargetCountersEnemy = 1;
		end

		if (Perl_Config.ShowPartyTarget == nil) then
			Perl_Config.ShowPartyTarget = 1;
	        end

		if (Perl_Config.PerlPlayerPortrait3D == nil) then
			Perl_Config.PerlPlayerPortrait3D = 1;
		end

		if (Perl_Config.PerlCombatFlash == nil) then
			Perl_Config.PerlCombatFlash = 1;
		end

		if (Perl_Config.PerlFadeFrames == nil) then
			Perl_Config.PerlFadeFrames = 1;
		end

		if (Perl_Config.TargetTargetHistory == nil) then
			Perl_Config.TargetTargetHistory = 1
		end

		if (Perl_Config.AssistsFrame == nil) then
			Perl_Config.AssistsFrame = 0;
		end

		if (Perl_Config.TargettingFrame == nil) then
			Perl_Config.TargettingFrame = 0;
		end

		if (Perl_Config.Scale_AssistsFrame == nil) then
			Perl_Config.Scale_AssistsFrame = 1;
		end

		if (Perl_Config.ShowPartyNumber == nil) then
			Perl_Config.ShowPartyNumber = 1;
		end

		Perl_Config.SortRaidByClass    		= nil;
		Perl_Config.ShowRaid	       		= nil;
		Perl_Config.ShowGroup1	       		= nil;
		Perl_Config.ShowGroup2	       		= nil;
		Perl_Config.ShowGroup3	   		= nil;
		Perl_Config.ShowGroup4			= nil;
		Perl_Config.ShowGroup5	       		= nil;
		Perl_Config.ShowGroup6	       		= nil;
		Perl_Config.ShowGroup7	       		= nil;
		Perl_Config.ShowGroup8	       		= nil;
		Perl_Config.ShowGroup9	       		= nil;
		Perl_Config.ShowRaidPercents   		= nil;
		Perl_Config.Scale_Raid	      		= nil;

		if (not Perl_Config.BackColour) then
			Perl_Config.BackColour = {["r"] = 0, ["g"] = 0, ["b"] = 0};
		end

		if (not Perl_Config.BorderColour) then
			Perl_Config.BorderColour = {["r"] = 0.5, ["g"] = 0.5, ["b"] = 0.5};
		end
	end

	if (myAddOnsFrame and not MCom) then
                myAddOnsList.Perl_Description = {
                        name			= Perl_Description,
                        description		= "Modifies the default party and target frames.",
                        version			= Perl_VersionNumber,
                        category		= MYADDONS_CATEGORY_OTHERS,
                        frame			= "Perl_Frame",
                        optionsframe		= "Perl_OptionsMenu_Frame"
                        };
	end

	Perl_Init();
        Perl_RegisterMCom();
end



-- Perl_InitFadeFrame(argFrame)
function Perl_InitFadeFrame(argFrame)
	argFrame.Fading = 0;
	argFrame.FadeTime = 0;
	argFrame.IsShowing = function () return (argFrame:IsVisible() and argFrame.Fading == 0); end;
end

function Perl_CancelFade(argFrame)
	if (argFrame.Fading == 1) then
		argFrame.Fading = 0;
		argFrame:SetAlpha(Perl_Config.Transparency);
	end
end

-- Perl_StartFade(argFrame)
function Perl_StartFade(argFrame)
	if (Perl_Config.PerlFadeFrames == 1) then
		argFrame:StopMovingOrSizing();

		if (argFrame:IsVisible() and argFrame.Fading == 0) then
			argFrame.Fading = 1;
			argFrame.FadeTime = 0;
		end
	else
		argFrame:Hide();
	end
end

-- Perl_ProcessFade(argFrame)
function Perl_ProcessFade(argFrame)
	if (argFrame.Fading == 1) then
		argFrame.FadeTime = argFrame.FadeTime + arg1;

		if (argFrame.FadeTime >= 1) then
			argFrame:StopMovingOrSizing();
			argFrame:Hide();
			Perl_CancelFade(argFrame);
		else
			local newAlpha = Perl_Config.Transparency * (1 - argFrame.FadeTime);
			argFrame:SetAlpha(newAlpha);
		end
	--else
	--	argFrame:SetAlpha(Perl_Config.Transparency);
	end
end
