local Perl_Player_Events = {}

---------------
-- Variables --
---------------

PerlColourTable = {
	[PERL_LOC_CLASS_HUNTER]		= "|c00AAD372",
	[PERL_LOC_CLASS_WARLOCK]	= "|c009382c9",
	[PERL_LOC_CLASS_PRIEST]		= "|c00FFFFFF",
	[PERL_LOC_CLASS_PALADIN]	= "|c00F48CBA",
	[PERL_LOC_CLASS_MAGE]		= "|c0068CCEF",
	[PERL_LOC_CLASS_ROGUE]		= "|c00FFF468",
	[PERL_LOC_CLASS_DRUID]		= "|c00FF7C0A",
	[PERL_LOC_CLASS_SHAMAN]		= "|c00F48CBA",
	[PERL_LOC_CLASS_WARRIOR]	= "|c00C69B6D"
};

----------------------
-- Loading Function --
----------------------

function Perl_Player_OnLoad()

	-- Events
	CombatFeedback_Initialize(Perl_PlayerHitIndicator, 30);
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");

	table.insert(UnitPopupFrames,"Perl_Player_DropDown");

	Perl_Player_DruidBar:SetHeight(1);
	Perl_Player_DruidBar:Hide();
	Perl_Player_DruidBarBG:SetHeight(1);
	Perl_Player_DruidBarBG:Hide();
	Perl_Player_CastClickOverlay:SetFrameLevel(Perl_Player_StatsFrame:GetFrameLevel() + 2);

	Perl_Player_Frame.PlayerFlash = 0;

	this.time = 0;
end

-------------------------
-- The Update Function --
-------------------------
function Perl_CombatFlashSet (elapsed, argFrame, argNew, argGreen)
	if (Perl_Config.PerlCombatFlash == 0) then
		argFrame.PlayerFlash = 0;
		argNew = false;
		return false;
	end

	if (argNew) then
		argFrame.PlayerFlash = 1;
		argFrame.PlayerFlashGreen = argGreen;
	else
		if (argFrame.PlayerFlash and elapsed) then
			argFrame.PlayerFlash = argFrame.PlayerFlash - elapsed;

			if (argFrame.PlayerFlash <= 0) then
				argFrame.PlayerFlash = 0;
				argFrame.PlayerFlashTime = nil;
				argFrame.PlayerFlashGreen = nil;
			end
		else
			return false;
		end
	end

	return true;
end

function Perl_CombatFlashSetFrames(argFrame, FlashIntensity, argGreen)
	if (argFrame and FlashIntensity) then
		if (FlashIntensity > 0) then
			local flashOffsetColour = FlashIntensity / 2;
			local r, g, b;
			if (argGreen) then
				--r = 0.5 - flashOffsetColour;
				--g = 0.5 + flashOffsetColour;
				--b = 0.5 - flashOffsetColour;
				r = Perl_Config.BorderColour.r - flashOffsetColour;
				g = Perl_Config.BorderColour.g + flashOffsetColour;
				b = Perl_Config.BorderColour.b - flashOffsetColour;
			else
				--r = 0.5 + flashOffsetColour;
				--g = 0.5 - flashOffsetColour;
				--b = 0.5 - flashOffsetColour;
				r = Perl_Config.BorderColour.r + flashOffsetColour;
				g = Perl_Config.BorderColour.g - flashOffsetColour;
				b = Perl_Config.BorderColour.b - flashOffsetColour;
			end

			if (r < 0) then r = 0; elseif (r > 1) then r = 1; end
			if (g < 0) then g = 0; elseif (g > 1) then g = 1; end
			if (b < 0) then b = 0; elseif (b > 1) then b = 1; end

			argFrame:SetBackdropBorderColor(r, g, b, 1);
		else
			argFrame:SetBackdropBorderColor(Perl_Config.BorderColour.r, Perl_Config.BorderColour.g, Perl_Config.BorderColour.b, 1);
		end
	end
end

local function Perl_Player_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Player_PortraitFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function PerlSetPortrait3D(argFrame, argUnit)
	argFrame:ClearModel();
	argFrame:SetUnit(argUnit);
	argFrame:SetCamera(0);
	--PerlPlayerPortrait3D:SetSequence(PerlSequence);
	--PerlPlayerPortrait3D:SetPosition(16,0,-0.65);
end

function Perl_Player_UpdatePortrait()
	if (Perl_Config.ShowPlayerPortrait == 1) then
		if (Perl_Config.ShowPlayerPortrait3D == 1) then
			PerlPlayerPortrait:Hide();
			PerlPlayerPortrait3D:Show();
			PerlSetPortrait3D(PerlPlayerPortrait3D, "player");
		else
			PerlPlayerPortrait:Show();
			PerlPlayerPortrait3D:Hide();
			SetPortraitTexture(PerlPlayerPortrait, "player");
		end
	end
end

local function Perl_Player_UpdateManaType()
	Perl_SetManaBarType("player", Perl_Player_ManaBar, Perl_Player_ManaBarBG);
end

function Perl_SetManaBarType(argUnit, argFrame, argFrameBG)
	local power = UnitPowerType(argUnit);

	local r, g, b;
	if (power == 1) then
		r, g, b = 1, 0, 0;
	elseif (power == 2) then
		r, g, b = 1, 0.5, 0;
	elseif (power == 3) then
		r, g, b = 1, 1, 0;
	else
		r, g, b = 0, 0, 1;
	end

	argFrame:SetStatusBarColor(r, g, b, 1);
	argFrameBG:SetStatusBarColor(r, g, b, 0.25);
end

function Perl_Player_UpdateLeader()
	if (IsPartyLeader()) then
		Perl_Player_LeaderIcon:Show();
	else
		Perl_Player_LeaderIcon:Hide();
	end

	if (Perl_Config.ShowPartyNumber == 1 and UnitName("raid1")) then
		for i = 1,GetNumRaidMembers() do
			local name, rank, subgroup = GetRaidRosterInfo(i);
			if (name == UnitName("player")) then
				Perl_Player_Group:SetText("Group "..subgroup);
				Perl_Player_GroupFrame:Show();
				return;
			end
		end
	end

	Perl_Player_GroupFrame:Hide();
end

local function Perl_Player_UpdateMasterLooter()
	local method, index = GetLootMethod();

	if (method == "master" and index == 0) then
		Perl_Player_MasterIcon:Show();
	else
		Perl_Player_MasterIcon:Hide();
	end
end

local function Perl_Player_UpdateName()
	Perl_Player_NameBarText:SetText(UnitName("player"));
end

local function Perl_Player_UpdateXP()
	local playerxp = UnitXP("player");
	local playerxpmax = UnitXPMax("player");
	local playerxprest = GetXPExhaustion();
	Perl_Player_XPBar:SetMinMaxValues(0, playerxpmax);
	Perl_Player_XPRestBar:SetMinMaxValues(0, playerxpmax);
	Perl_Player_XPBar:SetValue(playerxp);
	local xptext = playerxp.."/"..playerxpmax;
	if (playerxprest) then
		xptext = xptext .."(+"..(playerxprest)..")";
		Perl_Player_XPBar:SetStatusBarColor(0.3, 0.3, 1, 1);
		Perl_Player_XPRestBar:SetStatusBarColor(0.3, 0.3, 1, 0.5);
		Perl_Player_XPBarBG:SetStatusBarColor(0.3, 0.3, 1, 0.25);
		Perl_Player_XPRestBar:SetValue(playerxp + playerxprest);
	else
		Perl_Player_XPBar:SetStatusBarColor(0.6, 0, 0.6, 1);
		Perl_Player_XPRestBar:SetStatusBarColor(0.6, 0, 0.6, 0.5);
		Perl_Player_XPBarBG:SetStatusBarColor(0.6, 0, 0.6, 0.25);
		Perl_Player_XPRestBar:SetValue(playerxp);
	end
	Perl_Player_XPBarText:SetText(xptext);
	local xppercenttext=((playerxp * 100.0) / playerxpmax);
	xppercenttext=string.format("%3.0f", xppercenttext);
	Perl_Player_XPBarPercentText:SetText(xppercenttext.."%");
end

local function Perl_Player_UpdateCombat()
	if (UnitAffectingCombat("player")) then
		Perl_Player_NameBarText:SetTextColor(1,0,0);
		Perl_Player_ActivityStatus:SetTexCoord(0.5, 1.0, 0.0, 0.5);
		Perl_Player_ActivityStatus:Show();
	else
		if (UnitIsPVP("player")) then
			Perl_Player_NameBarText:SetTextColor(0,1,0);
		else
			Perl_Player_NameBarText:SetTextColor(0.5,0.5,1);
		end

		if (IsResting()) then
			Perl_Player_ActivityStatus:SetTexCoord(0, 0.5, 0.0, 0.5);
			Perl_Player_ActivityStatus:Show();
		else
			Perl_Player_ActivityStatus:Hide();
		end
	end
        Perl_Player_Pet_UpdateCombat();
end

local function Perl_Player_UpdatePVP()
	local playerrankname, playerrank=GetPVPRankInfo(UnitPVPRank("player"), "player");
	if (playerrank and Perl_Config.ShowPlayerPVPRank==1) then
		Perl_Player_PVPRankIcon:Show();
		if (playerrank==0) then
			Perl_Player_PVPRankIcon:Hide();
		elseif (playerrank>9) then
			Perl_Player_PVPRankIcon:SetTexture("Interface\\PVPRankBadges\\PVPRank"..playerrank);
		else
			Perl_Player_PVPRankIcon:SetTexture("Interface\\PVPRankBadges\\PVPRank0"..playerrank);
		end
	else
		Perl_Player_PVPRankIcon:Hide();
	end

	if (Perl_Config.ShowPlayerClassIcon) then
		local PlayerClass = UnitClass("player");
		Perl_Player_ClassTexture:SetTexCoord(Perl_ClassPosRight(PlayerClass), Perl_ClassPosLeft(PlayerClass), Perl_ClassPosTop(PlayerClass), Perl_ClassPosBottom(PlayerClass));
		--Perl_Player_ClassTexture:SetBackdropColor(Perl_Config.BackColour.r, Perl_Config.BackColour.g, Perl_Config.BackColour.b, Perl_Config.Transparency);
	end

	-- PVP Status settings
	if (UnitAffectingCombat("player")) then
		Perl_Player_NameBarText:SetTextColor(1,0,0);
	elseif (UnitIsPVP("player")) then
		Perl_Player_NameBarText:SetTextColor(0,1,0);
		Perl_Player_PVPStatus:Show();
		Perl_Player_PVPStatus:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..UnitFactionGroup("player"));
	else
		Perl_Player_NameBarText:SetTextColor(0.5,0.5,1);
		Perl_Player_PVPStatus:Hide();
	end
end

local function Perl_Player_Feigning ()
	for buffnum=1,16 do
		if (UnitBuff("player", buffnum)) then
			if (strfind(strlower(UnitBuff("player", buffnum)), "feigndeath")) then
				return true;
			end
		end
	end
	return false;
end

local function Perl_Player_CheckDeadOrGhost()
	if (UnitIsDead("player")) then			--(playerhealth==-100) and (playerhealthmax>1)) then
		if (Perl_Player_Feigning ()) then
			Perl_Player_HealthBarTEXT:SetText("Feign Death");
			return false;
		else
			Perl_Player_HealthBarTEXT:SetText("Dead");
			return true;
		end
		Perl_Player_ManaBarTEXT:Hide();

	elseif (UnitIsGhost("player")) then		-- (playerhealth==1) and (playerhealthmax>1)) then
		Perl_Player_HealthBarTEXT:SetText("Ghost");
		Perl_Player_ManaBarTEXT:Hide();
		return true;
	end

	return false;
end

local function Perl_Player_DruidBarUpdate()
	Perl_Player_DruidBar:SetMinMaxValues(0,DruidBarKey.maxmana);
	Perl_Player_DruidBar:SetValue(DruidBarKey.keepthemana);
	manaPct = (DruidBarKey.keepthemana * 100.0) / DruidBarKey.maxmana;
	manaPct =  string.format("%3.0f", manaPct);
	Perl_Player_DruidBarText:SetText(manaPct.."%");
	Perl_Player_DruidBarTEXT:SetText(math.floor(DruidBarKey.keepthemana).."/"..DruidBarKey.maxmana);
	if (UnitPowerType("player")>0) then
		if Perl_Config.ShowPlayerXPBar==1 then Perl_Player_StatsFrame:SetHeight(60);
		else Perl_Player_StatsFrame:SetHeight(50); end
		Perl_Player_DruidBarTEXT:Show();
		Perl_Player_DruidBarText:Show();
		Perl_Player_DruidBar:Show();
		Perl_Player_DruidBarBG:Show();
		Perl_Player_DruidBar:SetHeight(10);
		Perl_Player_DruidBarBG:SetHeight(10);
	else
		if Perl_Config.ShowPlayerXPBar==1 then Perl_Player_StatsFrame:SetHeight(50);
		else Perl_Player_StatsFrame:SetHeight(40); end
		Perl_Player_DruidBarTEXT:Hide();
		Perl_Player_DruidBarText:Hide();
		Perl_Player_DruidBar:Hide();
		Perl_Player_DruidBarBG:Hide();
		Perl_Player_DruidBar:SetHeight(1);
		Perl_Player_DruidBarBG:SetHeight(1);
	end

end

local function Perl_Player_UpdateMana()
	local playerhealth = UnitHealth("player");
	local playerhealthmax = UnitHealthMax("player");
	local playermana = UnitMana("player");
	local playermanamax = UnitManaMax("player");
	Perl_Player_ManaBar:SetMinMaxValues(0, playermanamax);
	Perl_Player_ManaBar:SetValue(playermana);
	manaPct = (playermana * 100.0) / playermanamax
	manaPct =  string.format("%3.0f", manaPct);
	Perl_Player_ManaBarTEXT:Show();
	Perl_Player_ManaBarText:Show();
	if (UnitPowerType("player")>=1) then
		Perl_Player_ManaBarText:SetText(playermana);
	else
		Perl_Player_ManaBarText:SetText(manaPct.."%");
	end
	Perl_Player_ManaBarTEXT:SetText(playermana.."/"..playermanamax);

	Perl_Player_CheckDeadOrGhost();

	if (DruidBarKey and (UnitClass("player")==PERL_LOC_CLASS_DRUID)) then
		Perl_Player_DruidBarUpdate();
	end
end

local function Perl_Player_UpdateHealth()
	local playerhealth = UnitHealth("player");
	local playerhealthmax = UnitHealthMax("player");
	Perl_Player_HealthBar:SetMinMaxValues(0, playerhealthmax);
	Perl_Player_HealthBar:SetValue(playerhealth);
	Perl_SetSmoothBarColor(Perl_Player_HealthBar);
	Perl_SetSmoothBarColor(Perl_Player_HealthBarBG, Perl_Player_HealthBar, 0.25);
	healthPct = (playerhealth * 100.0) / playerhealthmax
	healthPct = string.format("%3.0f", healthPct);
	Perl_Player_HealthBarTEXT:Show();
	Perl_Player_HealthBarText:SetText(healthPct.."%");
	Perl_Player_HealthBarTEXT:SetText(playerhealth.."/"..playerhealthmax);

	Perl_Player_CheckDeadOrGhost();
end

local function Perl_Player_UpdateLevel()
	Perl_Player_LevelFrame_LevelBarText:SetText(UnitLevel("player"));
end

function Perl_Player_OnUpdate(elapsed)

	if elapsed then
		Perl_Player_CombatFlash(elapsed, false);

		this.time = elapsed + this.time;
		if (this.time >= 0.3) then
		        Perl_Player_UpdateAssists();
			Perl_Target_ShowAssists();
			this.time = 0;
		end
	end

	--elseif this.time >= 5 then
	--	Perl_Player_UpdateDisplay();
		--if (this:GetScale() == UIParent:GetScale()) and (this:GetScale() ~= Perl_Config.Scale_PlayerFrame) then
		--	Perl_ScalePlayer(Perl_Config.Scale_PlayerFrame);
		--	Perl_ScalePet(Perl_Config.Scale_PetFrame);
		--	Perl_ScaleParty(Perl_Config.Scale_PartyFrame);
		--	Perl_ScaleTarget(Perl_Config.Scale_TargetFrame);
		--	Perl_ScaleTargetTarget(Perl_Config.Scale_TargetTargetFrame);
		--	Perl_ScaleRaid(Perl_Config.Scale_Raid);
		--end

end

local function Perl_Player_UpdateDisplay ()
	Perl_Player_UpdateXP();
	Perl_Player_UpdateManaType();
	if (Perl_Config.ShowPlayerLevel) then
		Perl_Player_UpdateLevel();
	end
	Perl_Player_UpdateName();
	Perl_Player_UpdatePVP();
	Perl_Player_UpdateCombat();
	Perl_Player_UpdateLeader();
	Perl_Player_UpdateMasterLooter();
	Perl_Player_UpdateMana();
	Perl_Player_UpdateHealth();
end

--------------------
-- Click Handlers --
--------------------

function Perl_PlayerDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_PlayerDropDown_Initialize, "MENU");
end

function Perl_PlayerDropDown_Initialize()
	UnitPopup_ShowMenu(Perl_Player_DropDown, "SELF", "player");
end
function Perl_Player_OnClick(button)
        if (not (CastPartyConfig and (Perl_Config.CastParty==1) and not (strfind(this:GetName(), ("Perl_Player_Frame"))))) then

		if ( SpellIsTargeting() and button == "RightButton" ) then
			SpellStopTargeting();
			return;
		end
		if ( button == "LeftButton" ) then
			if ( SpellIsTargeting() ) then
				SpellTargetUnit("player");
			elseif ( CursorHasItem() ) then
				DropItemOnUnit("player");
			else
				TargetUnit("player");
			end
		else
			ToggleDropDownMenu(1, nil, Perl_Player_DropDown, "Perl_Player_NameFrame", 40, 0);
		end

	end
        if (CastPartyConfig and (Perl_Config.CastParty==1) ) then
		--this:SetID(0);
		CastParty_OnClick(button);
	end
end
function Perl_Player_MouseUp(button)
	this:SetID(0);

	Perl_Player_Frame:StopMovingOrSizing();
end

function Perl_Player_MouseDown(button,param)
	if (param == "targetting") then
		Perl_Player_UpdateAssists();
		ToggleAssistsFrame("targetting");
		Perl_Target_ShowAssists();
	else
		if ( button == "LeftButton" and perl_locked == 0) then
			Perl_Player_Frame:StartMoving();
		end
	end
end

---------------------------------
-- Mouse enter/leave functions --
---------------------------------

--function Perl_Player_MouseEnter()
--end
--function Perl_Player_MouseLeave()
--end

function Perl_PlayerXPBar_MouseEnter()
	Perl_Player_XPBarText:Show();
end

function Perl_PlayerXPBar_MouseLeave()
	Perl_Player_XPBarText:Hide();
end

local function Perl_FoundEnemyBefore(FoundEnemy, name)
	for i = 1, table.getn(FoundEnemy) do
		local previous = FoundEnemy[i];
		if (UnitIsUnit(previous.."target", name.."target")) then
			return true;
		end
	end
	return false;
end

local function Perl_AddEnemy(anyEnemy, FoundEnemy, name)
	--if (UnitCanAttack(name.."targettarget", "player")) then
	if (UnitCanAttack("player", name.."target") and UnitIsUnit("player", name.."targettarget")) then
		if (not Perl_FoundEnemyBefore(FoundEnemy, name)) then
			table.insert (FoundEnemy, name);

			if (not anyEnemy) then
				Perl_Player_TargettingFrame.tooltip = Perl_Player_TargettingFrame.tooltip.."|c00FFFFFF"..PERL_TOOLTIP_ENEMYONME.."\13";
			end
			Perl_Player_TargettingFrame.tooltip = Perl_Player_TargettingFrame.tooltip.."|c00FF0000"..UnitName(name.."target").."\13";

			return true;
		end
	end
	return false;
end

---------------------------------
-- Targetting counters         --
---------------------------------
function Perl_Player_UpdateAssists()

	if (Perl_Config.TargetCounters == 0) then
		Perl_Target_AssistFrame:Hide();
		Perl_Player_TargettingFrame:Hide();
		return;
	end

	local assists = 0;
	local targetting = 0;
	local anyEnemy = false;
	local FoundEnemy = {};
	local start,i,total,prefix,name,petname,raidMembers;
	local targetname;
	raidMembers = 0;

	if (UnitInRaid("player")) then
		start = 1;
		prefix = "raid";
		total = GetNumRaidMembers();
	else
		start = 0;
		prefix = "party";
		total = 4;
	end

	Perl_Target_AssistFrame.tooltip = "";
	Perl_Player_TargettingFrame.tooltip = "";

	if (Perl_Config.AssistsFrame == 1) then
		Perl_Target_AssistFrame.assists = {};
	end
	if (Perl_Config.TargettingFrame == 1) then
		Perl_Target_AssistFrame.targetting = {};
	end

	targetname = UnitName("target");
	for i = start,total do
		if (i == 0) then
			name = "player";
		else
			name = prefix..i;
		end

		if (UnitName(name) and not UnitIsDeadOrGhost(name)) then
			local class = UnitClass(name);
			raidMembers = raidMembers + 1;

			if (targetname) then
				if (UnitIsUnit("target", name.."target")) then
					assists = assists + 1;

					if (Perl_Config.AssistsFrame == 1) then
						table.insert (Perl_Target_AssistFrame.assists, {UnitName(name), class});
					end

					if (Perl_Target_AssistFrame.tooltip == "") then
						Perl_Target_AssistFrame.tooltip = "|c00FFFFFF"..PERL_TOOLTIP_ASSISTING.."\13";
					end

					Perl_Target_AssistFrame.tooltip = Perl_Target_AssistFrame.tooltip..PerlColourTable[class]..UnitName(name).."\13";
				end
			end

			-- 0 for Anyone, 1 for Healers
			if (Perl_Config.TargetCountersSelf == 0 or (class == PERL_LOC_CLASS_PRIEST or class == PERL_LOC_CLASS_SHAMAN or class == PERL_LOC_CLASS_PALADIN or class == PERL_LOC_CLASS_DRUID)) then
				if (UnitIsUnit("player", name.."target")) then
					targetting = targetting + 1;

					if (Perl_Config.TargettingFrame == 1) then
						table.insert(Perl_Target_AssistFrame.targetting, {UnitName(name), class});
					end

					if (Perl_Player_TargettingFrame.tooltip == "") then
						if (Perl_Config.TargetCountersSelf == 0) then
							Perl_Player_TargettingFrame.tooltip = "|c00FFFFFF"..PERL_TOOLTIP_ALLONME.."\13";
						else
							Perl_Player_TargettingFrame.tooltip = "|c00FFFFFF"..PERL_TOOLTIP_HEALERS.."\13";
						end
					end

					Perl_Player_TargettingFrame.tooltip = Perl_Player_TargettingFrame.tooltip..PerlColourTable[class]..UnitName(name).."\13";
				end
			end
		end
	end

	if (Perl_Config.TargetCountersEnemy) then		-- Count enemy targetting us?
		for i = start,total do
			if (i == 0) then
				name = "player";
				petname = "pet";
			else
				name = prefix..i;
				petname = prefix.."pet"..i;
			end

			if (UnitExists(name)) then
				if (Perl_AddEnemy(anyEnemy, FoundEnemy, name)) then
					anyEnemy = true;
					targetting = targetting + 1;
					if (Perl_Config.TargettingFrame == 1) then
						table.insert(Perl_Target_AssistFrame.targetting, {UnitName(name.."target"), ""});
					end
				end
				if (UnitExists(petname)) then
					if (Perl_AddEnemy(anyEnemy, FoundEnemy, petname)) then
						anyEnemy = true;
						targetting = targetting + 1;
						if (Perl_Config.TargettingFrame == 1) then
							table.insert(Perl_Target_AssistFrame.targetting, {UnitName(petname.."target"), ""});
						end
					end
				end
			end
		end
	end

	if (anyEnemy) then
		Perl_Player_TargettingFrame:SetBackdropBorderColor(1, 0.2, 0.2, 1);
	else
		Perl_Player_TargettingFrame:SetBackdropBorderColor(Perl_Config.BorderColour.r, Perl_Config.BorderColour.g, Perl_Config.BorderColour.b, 1);
	end

	if (assists < 2) then
		Perl_Target_AssistBarText:SetTextColor(1,0.5,0.5,1);
	elseif (assists > (raidMembers / 2)) then
		Perl_Target_AssistBarText:SetTextColor(0.5,1,0.5,1);
	else
		Perl_Target_AssistBarText:SetTextColor(0.5,0.5,1,1);
	end

	if (targetting == 0) then
		Perl_Player_TargettingBarText:SetTextColor(1,0.5,0.5,1);
	elseif (targetting > 5) then
		Perl_Player_TargettingBarText:SetTextColor(0.5,1,0.5,1);
	else
		Perl_Player_TargettingBarText:SetTextColor(0.5,0.5,1,1);
	end

	if (targetname) then
		Perl_Target_AssistBarText:SetText(assists);
	end
	Perl_Player_TargettingBarText:SetText(targetting);

	--if (raidMembers > 1) then
	Perl_Target_AssistFrame:Show();
	Perl_Player_TargettingFrame:Show();
	--else
	--        Perl_Target_AssistFrame:Hide();
	--        Perl_Player_TargettingFrame:Hide();
	--end
end

-- EVENTS AND STUFF



function Perl_RegisterBasics(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:RegisterEvent("UNIT_RAGE");
	argFrame:RegisterEvent("UNIT_MAXRAGE");
	argFrame:RegisterEvent("UNIT_ENERGY");
	argFrame:RegisterEvent("UNIT_MAXENERGY");
	argFrame:RegisterEvent("UNIT_MANA");
	argFrame:RegisterEvent("UNIT_MAXMANA");
	argFrame:RegisterEvent("UNIT_HEALTH");
	argFrame:RegisterEvent("UNIT_MAXHEALTH");
	argFrame:RegisterEvent("UNIT_LEVEL");
	argFrame:RegisterEvent("UNIT_COMBAT");
	argFrame:RegisterEvent("UNIT_DISPLAYPOWER");
	argFrame:RegisterEvent("UNIT_NAME_UPDATE");
end
function Perl_UnregisterBasics(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:UnregisterEvent("UNIT_RAGE");
	argFrame:UnregisterEvent("UNIT_MAXRAGE");
	argFrame:UnregisterEvent("UNIT_ENERGY");
	argFrame:UnregisterEvent("UNIT_MAXENERGY");
	argFrame:UnregisterEvent("UNIT_MANA");
	argFrame:UnregisterEvent("UNIT_MAXMANA");
	argFrame:UnregisterEvent("UNIT_HEALTH");
	argFrame:UnregisterEvent("UNIT_MAXHEALTH");
	argFrame:UnregisterEvent("UNIT_LEVEL");
	argFrame:UnregisterEvent("UNIT_COMBAT");
	argFrame:UnregisterEvent("UNIT_DISPLAYPOWER");
	argFrame:UnregisterEvent("UNIT_NAME_UPDATE");
end

local function Perl_Player_RegisterSome()
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent("PARTY_LEADER_CHANGED");
	this:RegisterEvent("PARTY_LOOT_METHOD_CHANGED");
	this:RegisterEvent("RAID_ROSTER_UPDATE");
	this:RegisterEvent("UNIT_SPELLMISS");
	this:RegisterEvent("PLAYER_UPDATE_RESTING");
	this:RegisterEvent("UNIT_PVP_UPDATE");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	this:RegisterEvent("PLAYER_DEAD");
	this:RegisterEvent("PLAYER_FLAGS_CHANGED");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("PLAYER_XP_UPDATE");
	Perl_RegisterBasics();
end
local function Perl_Player_UnregisterSome()
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	this:UnregisterEvent("PARTY_LEADER_CHANGED");
	this:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED");
	this:UnregisterEvent("RAID_ROSTER_UPDATE");
	this:UnregisterEvent("UNIT_SPELLMISS");
	this:UnregisterEvent("PLAYER_UPDATE_RESTING");
	this:UnregisterEvent("UNIT_PVP_UPDATE");
	this:UnregisterEvent("UNIT_MODEL_CHANGED");
	this:UnregisterEvent("PLAYER_REGEN_ENABLED");
	this:UnregisterEvent("PLAYER_REGEN_DISABLED");
	this:UnregisterEvent("PLAYER_ENTER_COMBAT");
	this:UnregisterEvent("PLAYER_LEAVE_COMBAT");
	this:UnregisterEvent("PLAYER_LEVEL_UP");
	this:UnregisterEvent("PLAYER_DEAD");
	this:UnregisterEvent("PLAYER_FLAGS_CHANGED");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("PLAYER_XP_UPDATE");
	Perl_UnregisterBasics();
end

-------------------
-- Event Handler --
-------------------
function Perl_Player_OnEvent()
	local func = Perl_Player_Events[event];
	if (func) then
		func();

        	Perl_Target_Update_Combat();
        	for i = 1, 4 do
        	        Perl_Party_UpdateCombat1("party"..i, "Perl_party"..i);
        	end
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Player_Events:PLAYER_ENTERING_WORLD()
	Perl_Player_RegisterSome();
	Perl_Player_UpdateDisplay();
end

-- PLAYER_LEAVING_WORLD
function Perl_Player_Events:PLAYER_LEAVING_WORLD()
	Perl_Player_UnregisterSome();
end

-- UNIT_COMBAT
function Perl_Player_Events:UNIT_COMBAT()
	if (arg1 == "player") then
		CombatFeedback_OnCombatEvent(arg2, arg3, arg4, arg5);

		if (arg2 == "HEAL") then
			Perl_Player_CombatFlash(elapsed, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Player_CombatFlash(elapsed, true);
		end
	end
end

-- UNIT_SPELLMISS
function Perl_Player_Events:UNIT_SPELLMISS()
	if (arg1 == "player") then
		CombatFeedback_OnSpellMissEvent(arg2);
	end
end

-- UNIT_MODEL_CHANGED
function Perl_Player_Events:UNIT_MODEL_CHANGED()
	if (arg1 == "player") then
		Perl_Player_UpdatePortrait();
	end
end

-- VARIABLES_LOADED
function Perl_Player_Events:VARIABLES_LOADED()
	Perl_Player_UpdatePortrait();
	this:UnregisterEvent(event);
end

-- PARTY_LOOT_METHOD_CHANGED
function Perl_Player_Events:PARTY_LOOT_METHOD_CHANGED()
	Perl_Player_UpdateLeader();
	Perl_Player_UpdateMasterLooter();
end
Perl_Player_Events.PARTY_MEMBERS_CHANGED = Perl_Player_Events.PARTY_LOOT_METHOD_CHANGED;
Perl_Player_Events.PARTY_LEADER_CHANGED  = Perl_Player_Events.PARTY_LOOT_METHOD_CHANGED;
Perl_Player_Events.RAID_ROSTER_UPDATE    = Perl_Player_Events.PARTY_LOOT_METHOD_CHANGED;

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Player_Events:UNIT_HEALTH()
	if (arg1=="player") then
		Perl_Player_UpdateHealth();
	end
end
Perl_Player_Events.UNIT_MAXHEALTH = Perl_Player_Events.UNIT_HEALTH;
Perl_Player_Events.PLAYER_DEAD    = Perl_Player_Events.UNIT_HEALTH;

-- UNIT_MANA, UNIT_MAXMANA, UNIT_RAGE, UNIT_MAXRAGE, UNIT_ENERGY
-- UNIT_MAXENERGY, UNIT_FOCUS, UNIT_MAXFOCUS
function Perl_Player_Events:UNIT_MANA()
	if (arg1=="player") then
		Perl_Player_UpdateMana();
	end
end
Perl_Player_Events.UNIT_MAXMANA		= Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_RAGE		= Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_MAXRAGE         = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_ENERGY          = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_MAXENERGY       = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_FOCUS           = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_MAXFOCUS	= Perl_Player_Events.UNIT_MANA;

-- UNIT_DISPLAYPOWER
function Perl_Player_Events:UNIT_DISPLAYPOWER()
	Perl_Player_UpdateManaType();
	Perl_Player_UpdateMana();
end

-- UNIT_NAME_UPDATE
function Perl_Player_Events:UNIT_NAME_UPDATE()
	if (arg1=="player") then
		Perl_Player_UpdateName();
	end
end

-- UNIT_LEVEL
function Perl_Player_Events:UNIT_LEVEL()
	if (arg1 == "player") then
		if (Perl_Config.ShowPlayerLevel) then
			Perl_Player_UpdateLevel()
			Perl_Player_UpdateXP();
		end
	end
end

Perl_Player_Events.PLAYER_LEVEL_UP = Perl_Player_Events.UNIT_LEVEL;

-- PLAYER_XP_UPDATE
function Perl_Player_Events:PLAYER_XP_UPDATE()
	Perl_Player_UpdateXP();
end

-- UNIT_PVP_UPDATE
function Perl_Player_Events:UNIT_PVP_UPDATE()
	if (arg1 == "player") then
		Perl_Player_UpdatePVP();
	end
end

Perl_Player_Events.UNIT_FLAGS = Perl_Player_Events.UNIT_PVP_UPDATE;

function Perl_Player_Events:PLAYER_FLAGS_CHANGED()
	Perl_Player_UpdatePVP();
end

-- PLAYER_ENTER_COMBAT, PLAYER_LEAVE_COMBAT
function Perl_Player_Events:PLAYER_ENTER_COMBAT()
	Perl_Player_UpdateCombat();
end
Perl_Player_Events.PLAYER_LEAVE_COMBAT = Perl_Player_Events.PLAYER_ENTER_COMBAT;

-- PLAYER_REGEN_ENABLED, PLAYER_REGEN_DISABLED
function Perl_Player_Events:PLAYER_REGEN_ENABLED()
	Perl_Player_UpdateCombat();
	--Perl_Player_UpdateDisplay();
end
Perl_Player_Events.PLAYER_REGEN_DISABLED = Perl_Player_Events.PLAYER_REGEN_ENABLED;
Perl_Player_Events.PLAYER_UPDATE_RESTING = Perl_Player_Events.PLAYER_REGEN_ENABLED;
