local Perl_Player_Pet_Events = {}

----------------------
-- Loading Function --
----------------------
function Perl_Player_Pet_OnLoad()

	-- Events
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");

	table.insert(UnitPopupFrames,"Perl_Player_Pet_DropDown");

	this.time = 0;
	Perl_InitFadeFrame(this);
end

-------------------
-- Event Handler --
-------------------
local function Perl_Player_Pet_RegisterSome()
	this:RegisterEvent("UNIT_PET");
	this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("PET_STABLE_UPDATE");
	this:RegisterEvent("PET_STABLE_SHOW");
	this:RegisterEvent("PET_STABLE_CLOSED");
	this:RegisterEvent("PET_STABLE_UPDATE_PAPERDOLL");
end
local function Perl_Player_Pet_RegisterForPet()
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_FOCUS");
	this:RegisterEvent("UNIT_MAXFOCUS");
	this:RegisterEvent("PET_ATTACK_START");
	this:RegisterEvent("UNIT_PVP_UPDATE");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	this:RegisterEvent("UNIT_PET_EXPERIENCE");
	Perl_RegisterBasics();
end

local function Perl_Player_Pet_UnregisterSome()
	this:UnregisterEvent("UNIT_PET");
	this:UnregisterEvent("UNIT_HAPPINESS");
	this:UnregisterEvent("PET_STABLE_UPDATE");
	this:UnregisterEvent("PET_STABLE_SHOW");
	this:UnregisterEvent("PET_STABLE_CLOSED");
	this:UnregisterEvent("PET_STABLE_UPDATE_PAPERDOLL");
end
local function Perl_Player_Pet_UnregisterForPet()
	this:UnregisterEvent("UNIT_AURA");
	this:UnregisterEvent("UNIT_FOCUS");
	this:UnregisterEvent("UNIT_MAXFOCUS");
	this:UnregisterEvent("PET_ATTACK_START");
	this:UnregisterEvent("UNIT_PVP_UPDATE");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	this:UnregisterEvent("UNIT_PET_EXPERIENCE");
	Perl_UnregisterBasics();
end

-------------------------
-- The Update Function --
-------------------------
local function Perl_Player_Pet_CheckPet()
	if (UnitName("pet")) then
		if (UnitIsVisible("pet")) then
			Perl_CancelFade(Perl_Player_Pet_Frame);

			Perl_Player_Pet_Frame:Show();
			Perl_Player_Pet_RegisterForPet();
			return true;
		end
	end

	Perl_Player_Pet_UnregisterForPet();

	Perl_StartFade(Perl_Player_Pet_Frame);
	return false;
end

local function Perl_Player_Pet_UpdateName()
	local petname = UnitName("pet");

	if (petname == UNKNOWNOBJECT) then
		Perl_Player_Pet_NameBarText:SetText("");
	else
		Perl_Player_Pet_NameBarText:SetText(petname);
	end

	if (UnitIsPVP("pet")) then
	        Perl_Player_Pet_NameBarText:SetTextColor(0,1,0);
	else
	        Perl_Player_Pet_NameBarText:SetTextColor(0.5,0.5,1);
	end
end

local function Perl_Player_Pet_UpdateLevel()
	local petlevel = UnitLevel("pet");
	local petxp, petxpmax = GetPetExperience();

	Perl_Player_Pet_XPBar:SetStatusBarColor(0.3, 0.3, 1, 1);
	Perl_Player_Pet_XPBarBG:SetStatusBarColor(0.3, 0.3, 1, 0.25);

	Perl_Player_Pet_XPBar:SetMinMaxValues(0, petxpmax);
	Perl_Player_Pet_XPBar:SetValue(petxp);

	if (Perl_Config.ShowPetXP==0) then
		Perl_Player_Pet_XPBar:Hide();
		Perl_Player_Pet_XPBarBG:Hide();
		Perl_Player_Pet_StatsFrame:SetHeight(34);
	else
		Perl_Player_Pet_XPBar:Show();
		 Perl_Player_Pet_XPBarBG:Show();
		Perl_Player_Pet_StatsFrame:SetHeight(44);
	end

        Perl_Player_Pet_LevelBarText:SetText(petlevel);
end

local function Perl_Player_Pet_UpdateHealth()
	local pethealth = UnitHealth("pet");
	local pethealthmax = UnitHealthMax("pet");

	Perl_Player_Pet_HealthBar:SetMinMaxValues(0, pethealthmax);
	Perl_Player_Pet_HealthBar:SetValue(pethealth);

	Perl_SetSmoothBarColor(Perl_Player_Pet_HealthBar);
	Perl_SetSmoothBarColor(Perl_Player_Pet_HealthBarBG, Perl_Player_Pet_HealthBar, 0.25);

	if (UnitIsDead("pet")) then
		Perl_Player_Pet_HealthBarText:SetText("Dead");
		Perl_Player_Pet_ManaBarText:Hide();
	else
		Perl_Player_Pet_HealthBarText:SetText(pethealth.."/"..pethealthmax);
	end
end

local function Perl_Player_Pet_UpdateManaType()
	Perl_SetManaBarType("pet", Perl_Player_Pet_ManaBar, Perl_Player_Pet_ManaBarBG);
end

local function Perl_Player_Pet_UpdateMana()
	local petmana = UnitMana("pet");
	local petmanamax = UnitManaMax("pet");

	Perl_Player_Pet_ManaBar:SetMinMaxValues(0, petmanamax);
	Perl_Player_Pet_ManaBar:SetValue(petmana);

	Perl_Player_Pet_ManaBarText:SetText(petmana.."/"..petmanamax);

	Perl_Player_Pet_ManaBarText:Show();
end

local function Perl_Player_Pet_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Player_Pet_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_HappyFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function Perl_Player_Pet_OnUpdate(elapsed)
	if elapsed then
		Perl_Player_Pet_CombatFlash(elapsed, false);
	end

	Perl_ProcessFade(Perl_Player_Pet_Frame);
end

--------------------
-- Buff Functions --
--------------------
local function Perl_Player_Pet_Buff_UpdateAll ()
	if (UnitName("pet")) then
		for buffnum=1,10 do
			local button = getglobal("Perl_Player_Pet_Buff"..buffnum);
			local icon = getglobal(button:GetName().."Icon");
			if (UnitBuff("pet", buffnum)) then
				icon:SetTexture(UnitBuff("pet", buffnum));
				button:Show();
			else
				button:Hide();
			end
		end
		for buffnum=1,8 do
			local button = getglobal("Perl_Player_Pet_DeBuff"..buffnum);
			local icon = getglobal(button:GetName().."Icon");
			if (UnitDebuff("pet", buffnum)) then
				GameTooltip:SetUnitDebuff("pet", buffnum);
				icon:SetTexture(UnitDebuff("pet", buffnum));
				button:Show();
			else
				button:Hide();
			end
		end
	end
end

function Perl_Player_Pet_SetBuffTooltip ()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitBuff("pet", this:GetID());
end

function Perl_Player_Pet_SetDeBuffTooltip ()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitDebuff("pet", this:GetID());
end

---------------
-- Happiness --
---------------

local function Perl_Player_Pet_SetHappiness()
	local happiness, damagePercentage, loyaltyRate = GetPetHappiness();

	if ( happiness == 1 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0.375, 0.5625, 0, 0.359375);
	elseif ( happiness == 2 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0.1875, 0.375, 0, 0.359375);
	elseif ( happiness == 3 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0, 0.1875, 0, 0.359375);
	end
	if ( happiness ~= nil ) then
		Perl_Player_PetHappiness.tooltip = getglobal("PET_HAPPINESS"..happiness);
		Perl_Player_PetHappiness.tooltipDamage = format(PET_DAMAGE_PERCENTAGE, damagePercentage);
		if ( loyaltyRate < 0 ) then
			Perl_Player_PetHappiness.tooltipLoyalty = getglobal("LOSING_LOYALTY");
		elseif ( loyaltyRate > 0 ) then
			Perl_Player_PetHappiness.tooltipLoyalty = getglobal("GAINING_LOYALTY");
		else
			Perl_Player_PetHappiness.tooltipLoyalty = nil;
		end
	end
end

--------------------
-- Click Handlers --
--------------------

function Perl_Player_Pet_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_Player_Pet_DropDown_Initialize, "MENU");
end

function Perl_Player_Pet_DropDown_Initialize ()
	UnitPopup_ShowMenu(Perl_Player_Pet_DropDown, "PET", "pet");
end

function Perl_Player_Pet_MouseUp(button)

	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end
	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			SpellTargetUnit("pet");
		else
			TargetUnit("pet");
		end
	else
		ToggleDropDownMenu(1, nil, Perl_Player_Pet_DropDown, "Perl_Player_Pet_NameFrame", 40, 0);
	end

	Perl_Player_Pet_Frame:StopMovingOrSizing();
end

function Perl_Player_Pet_MouseDown(button)
	if ( button == "LeftButton" and perl_locked == 0) then
		Perl_Player_Pet_Frame:StartMoving();
	end
end

local function Perl_Target_Update_Control()
        if (UnitIsCharmed("pet")) then
		Perl_Player_Pet_Warning:Show();
	else
		Perl_Player_Pet_Warning:Hide();
	end
end

function Perl_Player_Pet_UpdateCombat()
	if (UnitName("pet")) then
                if (UnitAffectingCombat("pet")) then
                        Perl_Player_Pet_ActivityStatus:SetTexCoord(0.5, 1.0, 0.0, 0.5);
                        Perl_Player_Pet_ActivityStatus:Show();
                else
                        Perl_Player_Pet_ActivityStatus:Hide();
                end
		Perl_Target_Update_Control();
        end
end

local function Perl_Player_Pet_UpdateDisplay ()
	if (Perl_Player_Pet_CheckPet()) then
		Perl_CancelFade(Perl_Player_Pet_Frame);

		Perl_Player_Pet_UpdateName();
		Perl_Player_Pet_UpdateHealth();
		Perl_Player_Pet_UpdateManaType();
		Perl_Player_Pet_UpdateMana();
		Perl_Player_Pet_UpdateLevel();
		Perl_Player_Pet_SetHappiness();
		Perl_Player_Pet_Buff_UpdateAll();

	        Perl_Player_Pet_UpdateCombat();
	end
end

-------------------
-- Event Handler --
-------------------
function Perl_Player_Pet_OnEvent()
	local func = Perl_Player_Pet_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Player_Pet_Events:PLAYER_ENTERING_WORLD()
	Perl_Player_Pet_Frame.PlayerFlash = 0;
	Perl_Player_Pet_RegisterSome();
end

-- PLAYER_LEAVING_WORLD
function Perl_Player_Pet_Events:PLAYER_LEAVING_WORLD()
	Perl_Player_Pet_UnregisterSome();
end

-- UNIT_AURA
function Perl_Player_Pet_Events:UNIT_AURA()
	if (arg1 == "pet") then
		Perl_Player_Pet_Buff_UpdateAll();
	end
end

-- UNIT_PET
function Perl_Player_Pet_Events:UNIT_PET()
	Perl_Player_Pet_UpdateDisplay();
end

Perl_Player_Pet_Events.PET_STABLE_SHOW = Perl_Player_Pet_Events.UNIT_PET;

-- UNIT_NAME_UPDATE
function Perl_Player_Pet_Events:UNIT_NAME_UPDATE()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateName();
	end
end

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Player_Pet_Events:UNIT_HEALTH()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateHealth();
	end
end
Perl_Player_Pet_Events.UNIT_MAXHEALTH = Perl_Player_Pet_Events.UNIT_HEALTH;

-- UNIT_RAGE
function Perl_Player_Pet_Events:UNIT_RAGE()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateMana();
	end
end

Perl_Player_Pet_Events.UNIT_MAXRAGE	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_ENERGY	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXENERGY	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MANA	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXMANA	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_FOCUS	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXFOCUS	= Perl_Player_Pet_Events.UNIT_RAGE;

-- UNIT_LEVEL
function Perl_Player_Pet_Events:UNIT_LEVEL()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateLevel();
	end
end

Perl_Player_Pet_Events.UNIT_PET_EXPERIENCE = Perl_Player_Pet_Events.UNIT_LEVEL;

-- UNIT_DISPLAYPOWER
function Perl_Player_Pet_Events:UNIT_DISPLAYPOWER()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateManaType();
	end
end

-- UNIT_HAPPINESS
-- Happiness events come while you have a pet up, each 2 secs
function Perl_Player_Pet_Events:UNIT_HAPPINESS()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateDisplay();	-- Update all, fixes bug with stables in 1.11
	--	Perl_Player_Pet_SetHappiness();
	--      Perl_Player_Pet_UpdateCombat();
	end
end

-- PET_ATTACK_START
function Perl_Player_Pet_Events:PET_ATTACK_START()
	Perl_Player_Pet_UpdateCombat();
end

-- UNIT_COMBAT
function Perl_Player_Pet_Events:UNIT_COMBAT()
	if (arg1 == "pet") then
		if (arg2 == "HEAL") then
			Perl_Player_Pet_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Player_Pet_CombatFlash(0, true);
		end
	end
end

-- UNIT_DYNAMIC_FLAGS
function Perl_Player_Pet_Events:UNIT_PVP_UPDATE()
	if (arg1 == "pet" or arg1 == "player") then
		Perl_Player_Pet_UpdateName();
	        Perl_Player_Pet_UpdateCombat();
	end
end

Perl_Player_Pet_Events.UNIT_FLAGS = Perl_Player_Pet_Events.UNIT_PVP_UPDATE;
Perl_Player_Pet_Events.UNIT_DYNAMIC_FLAGS = Perl_Player_Pet_Events.UNIT_PVP_UPDATE;
