local Perl_Target_Events = {}

----------------------
-- Loading Function --
----------------------

function Perl_Target_OnLoad()
	CombatFeedback_Initialize(Perl_TargetHitIndicator, 30);

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");

	--this:Hide();

	table.insert(UnitPopupFrames,"Perl_Target_DropDown");

	this.PlayerFlash = 0;
	this.lastTarget = "";
	this.lastDeathMsg = "";
	Perl_InitFadeFrame(this);
end

function Perl_Target_Assists_OnLoad()
	Perl_InitFadeFrame(this);

	--Perl_Target_Assists_Text:SetFontObject(GameFontNormalSmall);
	--Perl_Target_Targetting_Text:SetFontObject(GameFontNormalSmall);
end

local function Perl_Target_RegisterSome()
	this:RegisterEvent("UNIT_SPELLMISS");
	this:RegisterEvent("UNIT_FACTION");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_CLASSIFICATION_CHANGED");
	--this:RegisterEvent("PLAYER_PVPLEVEL_CHANGED");
	this:RegisterEvent("UNIT_PVP_UPDATE");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("PLAYER_FLAGS_CHANGED");
	this:RegisterEvent("PLAYER_COMBO_POINTS");
	this:RegisterEvent("UNIT_MODEL_CHANGED");		--UNIT_PORTRAIT_UPDATE");
	this:RegisterEvent("PARTY_MEMBER_DISABLE");
	this:RegisterEvent("PARTY_MEMBER_ENABLE");
	this:RegisterEvent("UNIT_AURA");

	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	Perl_RegisterBasics();
end
local function Perl_Target_UnregisterSome()
	this:UnregisterEvent("UNIT_SPELLMISS");
	this:UnregisterEvent("UNIT_FACTION");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_CLASSIFICATION_CHANGED");
	--this:UnregisterEvent("PLAYER_PVPLEVEL_CHANGED");
	this:UnregisterEvent("UNIT_PVP_UPDATE");
	this:UnregisterEvent("PLAYER_TARGET_CHANGED");
	this:UnregisterEvent("PLAYER_FLAGS_CHANGED");
	this:UnregisterEvent("PLAYER_COMBO_POINTS");
	this:UnregisterEvent("UNIT_MODEL_CHANGED");		--UNIT_PORTRAIT_UPDATE");
	this:UnregisterEvent("PARTY_MEMBER_DISABLE");
	this:UnregisterEvent("PARTY_MEMBER_ENABLE");
	this:UnregisterEvent("UNIT_AURA");

	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	Perl_UnregisterBasics();
end


--------------------
-- Buff Functions --
--------------------
function Perl_Targets_BuffUpdate(unitTarget, argFramePrefix)

	if (this.Fading == 1) then
		return;
	end

	local frameBuffs = getglobal(argFramePrefix.."BuffFrameBuffer");
	local frameDebuffs = getglobal(argFramePrefix.."DeBuffFrameBuffer");

	--Perl_Target_UpdatePortrait();
	if ((UnitFactionGroup("player") == UnitFactionGroup(unitTarget))) then
		frameDebuffs:SetHeight(24);
		frameBuffs:SetHeight(3);
	else
		frameDebuffs:SetHeight(3);
		if (UnitDebuff(unitTarget, 9)) then
			frameBuffs:SetHeight(58);
		else
			frameBuffs:SetHeight(34);
		end
	end

	if (UnitName(unitTarget)) then
		for buffnum=1,20 do
			local button = getglobal(argFramePrefix.."Buff"..buffnum);
			local icon = getglobal(button:GetName().."Icon");
			local buff, count = UnitBuff(unitTarget, buffnum);
			if (buff) then
				local buffCount = getglobal(argFramePrefix.."Buff"..buffnum.."Count");
				icon:SetTexture(buff);
				if (count > 1) then
					buffCount:SetText(count);
					buffCount:Show();
				else
					buffCount:Hide();
				end
				button:Show();
			else
				button:Hide();
			end
		end

		for buffnum=1,16 do
			local button = getglobal(argFramePrefix.."DeBuff"..buffnum);
			local icon = getglobal(button:GetName().."Icon");

			if (UnitDebuff(unitTarget, buffnum)) then
				local debuffCount = getglobal(argFramePrefix.."DeBuff"..buffnum.."Count");

				local debuff, debuffApplications = UnitDebuff(unitTarget, buffnum);
				icon:SetTexture(debuff);
				if ( debuffApplications > 1 ) then
					debuffCount:SetText(debuffApplications);
					debuffCount:Show();
				else
					debuffCount:Hide();
				end
				button:Show();
			else
				button:Hide();
			end

		end
	end
end

local function Perl_Target_Buff_UpdateAll ()
	Perl_Targets_BuffUpdate("target", "Perl_Target_");
end

function Perl_Target_SetBuffTooltip (target)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitBuff(target, this:GetID());
	Perl_ToolTip_AddBuffDuration(target);
end
function Perl_Target_SetDeBuffTooltip (target)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitDebuff(target, this:GetID());
	Perl_ToolTip_AddDebuffDuration(target);
end

function Perl_Target_PlayerTip()
	GameTooltip_SetDefaultAnchor(GameTooltip, this);
	GameTooltip:SetUnit("Target");
end

---------------------
--Debuffs          --
---------------------
local function Perl_Target_DebuffUpdate()
	if (this.Fading == 1) then
		return;
	end

	local numDebuffs = 0;

	if UnitClass("player")==PERL_LOC_CLASS_WARRIOR then
		numDebuffs = Perl_UnitDebuffInformation("Sunder Armor");
	elseif UnitClass("player")==PERL_LOC_CLASS_PRIEST then
		numDebuffs = Perl_UnitDebuffInformation("Shadow Vulnerability");
	elseif UnitClass("player")==PERL_LOC_CLASS_MAGE then
		numDebuffs = Perl_UnitDebuffInformation("Fire Vulnerability");
	end
	if (GetComboPoints()==0) then
		if (Perl_Config.UseCPMeter == 1) then
			Perl_Target_CPFrame:Hide();
			Perl_Target_CPMeter:SetMinMaxValues(0, 5);
			Perl_Target_CPMeter:SetValue(numDebuffs);
			Perl_Target_CPMeter:Show();
			if (numDebuffs == 5) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0, 0, 0.4);
			elseif (numDebuffs == 4) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0.5, 0, 0.4);
			elseif (numDebuffs == 3) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 1, 0, 0.4);
			elseif (numDebuffs == 2) then
				Perl_Target_CPMeter:SetStatusBarColor(0.5, 1, 0, 0.4);
			elseif (numDebuffs == 1) then
				Perl_Target_CPMeter:SetStatusBarColor(0, 1, 0, 0.4);
			else
				Perl_Target_CPMeter:Hide();
			end
		else
			Perl_Target_CPMeter:Hide();
			Perl_Target_CPFrame:Show();
			Perl_Target_CPText:Show();
			Perl_Target_CPText:SetText(numDebuffs);
			if (cnumDebuffs == 5) then
				Perl_Target_CPText:SetTextColor(1, 0, 0, 1);
			elseif (numDebuffs == 4) then
				Perl_Target_CPText:SetTextColor(1, 0.5, 0, 1);
			elseif (numDebuffs == 3) then
				Perl_Target_CPText:SetTextColor(1, 1, 0, 1);
			elseif (numDebuffs == 2) then
				Perl_Target_CPText:SetTextColor(0.5, 1, 0,1);
			elseif (numDebuffs == 1) then
				Perl_Target_CPText:SetTextColor(0, 1, 0, 1);
			else
				Perl_Target_CPText:Hide();
				Perl_Target_CPFrame:Hide();
			end
		end
	else
		local combopoints = GetComboPoints();
		if (Perl_Config.UseCPMeter == 1) then
			Perl_Target_CPFrame:Hide();
			Perl_Target_CPMeter:SetMinMaxValues(0, 5);
			Perl_Target_CPMeter:SetValue(combopoints);
			Perl_Target_CPMeter:Show();
			if (combopoints == 5) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0, 0, 0.7);
			elseif (combopoints == 4) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0.5, 0, 0.7);
			elseif (combopoints == 3) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 1, 0, 0.6);
			elseif (combopoints == 2) then
				Perl_Target_CPMeter:SetStatusBarColor(0.5, 1, 0, 0.4);
			elseif (combopoints == 1) then
				Perl_Target_CPMeter:SetStatusBarColor(0, 1, 0, 0.4);
			else
				Perl_Target_CPMeter:Hide();
			end
		else
			Perl_Target_CPMeter:Hide();
			Perl_Target_CPFrame:Show();
			Perl_Target_CPText:Show();
			Perl_Target_CPText:SetText(combopoints);
			if (combopoints == 5) then
				Perl_Target_CPText:SetTextColor(1, 0, 0, 1);
			elseif (combopoints == 4) then
				Perl_Target_CPText:SetTextColor(1, 0.5, 0, 1);
			elseif (combopoints == 3) then
				Perl_Target_CPText:SetTextColor(1, 1, 0, 1);
			elseif (combopoints == 2) then
				Perl_Target_CPText:SetTextColor(0.5, 1, 0,1);
			elseif (combopoints == 1) then
				Perl_Target_CPText:SetTextColor(0, 1, 0, 1);
			else
				Perl_Target_CPText:Hide();
				Perl_Target_CPFrame:Hide();
			end
		end
	end
end


-------------------------
-- The Update Functions--
-------------------------
local function Perl_Target_UpdatePVP()
	local targetrankname, targetrank=GetPVPRankInfo(UnitPVPRank("target"), "target");
	if (targetrank and Perl_Config.ShowTargetPVPRank==1 and UnitIsPlayer("target")) then
		Perl_Target_PVPRankIcon:Show();
		if (targetrank==0) then
			Perl_Target_PVPRankIcon:Hide();
		elseif (targetrank>9) then
			Perl_Target_PVPRankIcon:SetTexture("Interface\\PVPRankBadges\\PVPRank"..targetrank);
		else
			Perl_Target_PVPRankIcon:SetTexture("Interface\\PVPRankBadges\\PVPRank0"..targetrank);
		end
	else
		Perl_Target_PVPRankIcon:Hide();
	end
	if (UnitIsPVP("target")) then
		local p = "Interface\\TargetingFrame\\UI-PVP-";
		if (UnitFactionGroup("target") == "Alliance") then
			Perl_Target_PVPStatus:SetTexture(p.."Alliance");
		elseif (UnitFactionGroup("target") == "Horde") then
			Perl_Target_PVPStatus:SetTexture(p.."Horde");
		else
			Perl_Target_PVPStatus:SetTexture(p.."FFA");
		end
		Perl_Target_PVPStatus:Show();
	else
		Perl_Target_PVPStatus:Hide();
	end
end
--
local function Perl_Target_UpdateName()
	local targetname = UnitName("target");

	--if (strlen(targetname) > 20) then
	--	targetname = strsub(targetname, 1, 19).."...";
	--end

	Perl_Target_NameBarText:SetText(targetname);

	local remCount = 1;
	while ((Perl_Target_NameBarText:GetStringWidth() >= (Perl_Target_NameFrame:GetWidth() - 8)) and (string.len(targetname) > remCount)) do
		targetname = string.sub(targetname, 1, string.len(targetname) - remCount).."..";
		remCount = 3;
		Perl_Target_NameBarText:SetText(targetname);
	end

        Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
	Perl_Target_UpdatePVP();

	--guildname, guildtitle, guildrank = GetGuildInfo("target");
end

function Perl_SetUnitNameColor(argUnit,argFrame)

	local r, g, b = 0.5, 0.5, 1;

        if (UnitPlayerControlled(argUnit) or not UnitIsVisible(argUnit)) then
                if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
                        if (UnitIsEnemy("player", argUnit)) then
				-- Dueling
                                r, g, b = 1, 0, 0;

                        elseif (UnitIsPVP(argUnit)) then
                                r, g, b = 0, 1, 0;
			end
		else
                        if (UnitIsPVP(argUnit)) then
				if (UnitIsPVP("player")) then
					r, g, b = 1, 0, 0;
				else
					r, g, b = 1, 1, 0;
				end
			end
		end
	else
                if (UnitIsTapped(argUnit) and not UnitIsTappedByPlayer(argUnit)) then
                        r, g, b = 0.5, 0.5, 0.5;
                else
			local reaction = UnitReaction(argUnit, "player");

			if (reaction) then
				if (reaction >= 5) then
					r, g, b = 0, 1, 0;
				elseif (reaction <= 2) then
					r, g, b = 1, 0, 0;
				elseif (reaction == 3) then
					r, g, b = 1, 0.5, 0;
				else
					r, g, b = 1, 1, 0;
				end
			else
                                --argFrame:SetTextColor(0.5,0.5,1);
                        	if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
					r, g, b = 0, 1, 0;
                        	elseif (UnitIsEnemy("player", argUnit)) then
					r, g, b = 1, 0, 0;
                        	else
					r, g, b = 1, 1, 0;
                        	end
			end
		end
	end

	argFrame:SetTextColor(r, g, b);
end

function Perl_Target_UpdatePortrait()
	if (Perl_Config.ShowTargetPortrait == 1) then
		if (UnitName("target")) then
			if (UnitIsVisible("target") and Perl_Config.ShowPlayerPortrait3D == 1) then
				PerlTargetPortrait:Hide();
				PerlTargetPortrait3D:Show();
				PerlSetPortrait3D(PerlTargetPortrait3D, "target");
			else
				PerlTargetPortrait:Show();
				PerlTargetPortrait3D:Hide();
				SetPortraitTexture(PerlTargetPortrait, "target");
			end
		end
	end
end

local function Perl_Target_UpdateLevel()
	local targetlevel = UnitLevel("target");
	Perl_Target_LevelBarText:SetText(targetlevel);
	-- Set Level
	if (Perl_Config.ShowTargetLevel==1) then
		Perl_Target_LevelFrame:Show();
		Perl_Target_LevelFrame:SetWidth(27);
		if (targetlevel < 0) then
			Perl_Target_LevelBarText:SetTextColor(1, 0, 0);
			Perl_Target_LevelFrame:Hide();
		else
			local color = GetDifficultyColor(targetlevel);
			Perl_Target_LevelBarText:SetTextColor(color.r, color.g, color.b);
		end
                if ((Perl_Config.ShowTargetElite==0) and (UnitIsPlusMob("target"))) then
			Perl_Target_LevelBarText:SetText(targetlevel.."+");
			Perl_Target_LevelFrame:SetWidth(33);
		end
	end
end

local function Perl_Target_UpdateClassification()
	local targetclassification = UnitClassification("target");

	if (targetclassification == "normal" and UnitPlayerControlled("target")) then
		if (not UnitIsPlayer("target")) then
			-- Pet
			Perl_Target_CreatureBossText:SetText("Pet");
	                Perl_Target_CreatureBossText:SetTextColor(1, 1, 1);
			Perl_Target_BossFrame:SetWidth(38);
		        Perl_Target_BossFrame:Show();
			return;
		end
	        Perl_Target_BossFrame:Hide();
	else
	        if ((Perl_Config.ShowTargetLevel==1 and Perl_Config.ShowTargetElite==1) or targetclassification=="Rare+" or targetclassification=="Rare") then
			Perl_Target_BossFrame:Show();
			if (targetclassification == "worldboss") then
				Perl_Target_CreatureBossText:SetText("Boss");
	                        Perl_Target_CreatureBossText:SetTextColor(1, 0.5, 0.5);
				Perl_Target_BossFrame:SetWidth(38);
			elseif (targetclassification == "rareelite") then
				Perl_Target_CreatureBossText:SetText("Rare+");
	                        Perl_Target_CreatureBossText:SetTextColor(0.8, 0.8, 0.8);
				Perl_Target_BossFrame:SetWidth(43);
			elseif (targetclassification == "elite") then
				Perl_Target_CreatureBossText:SetText("Elite");
	                        Perl_Target_CreatureBossText:SetTextColor(1, 1, 0.5);
				Perl_Target_BossFrame:SetWidth(43);
			elseif (targetclassification == "rare") then
				Perl_Target_CreatureBossText:SetText("Rare");
	                        Perl_Target_CreatureBossText:SetTextColor(0.8, 0.8, 0.8);
				Perl_Target_BossFrame:SetWidth(38);
			else
				Perl_Target_BossFrame:Hide();
			end
	        else
	                Perl_Target_BossFrame:Hide();
		end
	end
end

local function Perl_Target_UpdateType()
	--if (UnitIsPlayer("target")) then
	--	if (UnitFactionGroup("player")~=UnitFactionGroup("target")) then
	--		Perl_Target_CreatureText:SetTextColor(1, 0, 0);
	--	else
	-- 		Perl_Target_CreatureText:SetTextColor(0.3, 0.3, 1);
	--	end
	--end

	local targettype = UnitCreatureType("target");
	if (Perl_Config.ShowTargetMobType==1) then Perl_Target_TypeFrame:Show(); end
	Perl_Target_TypeFramePlayer:Hide();
	Perl_Target_CreatureText:SetText(targettype);
	if (UnitIsPlayer("target")) then
		local PlayerClass = UnitClass("target");
		if (Perl_Config.ShowTargetClassIcon==1) then
			Perl_Target_ClassTexture:SetTexCoord(Perl_ClassPosRight(PlayerClass), Perl_ClassPosLeft(PlayerClass), Perl_ClassPosTop(PlayerClass), Perl_ClassPosBottom(PlayerClass));
			Perl_Target_TypeFramePlayer:Show();
		end
		Perl_Target_TypeFrame:Hide();

	 elseif (targettype == PERL_LOC_TYPE_HUMANOID) then
		 Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_HUMANOID_LENGTH);
 	elseif (targettype == PERL_LOC_TYPE_BEAST) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_BEAST_LENGTH);
 	elseif (targettype == PERL_LOC_TYPE_GIANT) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_GIANT_LENGTH);
	 elseif (targettype == PERL_LOC_TYPE_UNDEAD) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_UNDEAD_LENGTH);
 	elseif (targettype == PERL_LOC_TYPE_DEMON) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_DEMON_LENGTH);
	 elseif (targettype == PERL_LOC_TYPE_ELEMENTAL) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_ELEMENTAL_LENGTH);
 	elseif (targettype == PERL_LOC_TYPE_CRITTER) then
		 Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_CRITTER_LENGTH);
 	elseif (targettype == PERL_LOC_TYPE_DRAGONKIN) then
 		Perl_Target_TypeFrame:SetWidth(PERL_LOC_TYPE_DRAGONKIN_LENGTH);
	else
		Perl_Target_TypeFrame:Hide();
	end
	Perl_Target_CreatureText:SetTextColor(1, 1, 1);
end

local function Perl_Target_UpdateCombo()
	local combopoints = GetComboPoints();
	if (Perl_Config.UseCPMeter == 1) then
		Perl_Target_CPFrame:Hide();
		Perl_Target_CPMeter:SetMinMaxValues(0, 5);
		Perl_Target_CPMeter:SetValue(combopoints);
		Perl_Target_CPMeter:Show();
		if (combopoints == 5) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 0, 0, 0.7);
		elseif (combopoints == 4) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 0.5, 0, 0.7);
		elseif (combopoints == 3) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 1, 0, 0.6);
		elseif (combopoints == 2) then
			Perl_Target_CPMeter:SetStatusBarColor(0.5, 1, 0, 0.4);
		elseif (combopoints == 1) then
			Perl_Target_CPMeter:SetStatusBarColor(0, 1, 0, 0.4);
		else
			Perl_Target_CPMeter:Hide();
		end
	else
		Perl_Target_CPMeter:Hide();
		Perl_Target_CPFrame:Show();
		Perl_Target_CPText:Show();
		Perl_Target_CPText:SetText(combopoints);
		if (combopoints == 5) then
			Perl_Target_CPText:SetTextColor(1, 0, 0, 1);
		elseif (combopoints == 4) then
			Perl_Target_CPText:SetTextColor(1, 0.5, 0, 1);
		elseif (combopoints == 3) then
			Perl_Target_CPText:SetTextColor(1, 1, 0, 1);
		elseif (combopoints == 2) then
			Perl_Target_CPText:SetTextColor(0.5, 1, 0,1);
		elseif (combopoints == 1) then
			Perl_Target_CPText:SetTextColor(0, 1, 0, 1);
		else
			Perl_Target_CPText:Hide();
			Perl_Target_CPFrame:Hide();
		end
	end
end

local function Perl_Target_UpdateManaType()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetManaType("target", Perl_Target_ManaBar, Perl_Target_ManaBarText, Perl_Target_ManaBarBG, Perl_Target_StatsFrame, Perl_Target_Frame);
end
function Perl_Target_SetManaType(unit, frameManaBar, frameManaBarText, frameManaBarBG, frameStatsFrame, frameFrame)
	local targetpower = UnitPowerType(unit);
	local targetmana = UnitMana(unit);
	local targetmanamax = UnitManaMax(unit);
	local r, g, b;

	if ((targetmanamax == 0) or (Perl_Config.ShowTargetMana==0)) then
		frameManaBar:Hide();
		frameManaBarBG:Hide();
		frameStatsFrame:SetHeight(28);
		frameFrame:SetHeight(28);
		return;

	elseif (targetpower == 1) then
		r, g, b = 1, 0, 0;
	elseif (targetpower == 2) then
		r, g, b = 1, 0.5, 0;
	elseif (targetpower == 3) then
		r, g, b = 1, 1, 0;
	else
		r, g, b = 0, 0, 1;
		--frameManaBarText:Show();
	end

	frameManaBarText:Show();
	frameManaBar:Show();
	frameManaBarBG:Show();
	frameStatsFrame:SetHeight(40);
	frameFrame:SetHeight(50);
	frameManaBar:SetStatusBarColor(r, g, b, 1);
	frameManaBarBG:SetStatusBarColor(r, g, b, 0.25);
end

local function Perl_Target_UpdateMana()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetMana("target", Perl_Target_ManaBar, Perl_Target_HealthBarPercent, Perl_Target_ManaBarPercent, Perl_Target_ManaBarText);
end
function Perl_Target_SetMana(unit, frameManabar, frameHealthBarPercent, frameManaBarPercent, frameManaBarText)
	local targetmana = UnitMana(unit);
	local targetmanamax = UnitManaMax(unit);
	if (frameHealthBarPercent:IsVisible()) then
		frameManaBarPercent:Show();
	end

	if (targetmanamax == 1 and targetmana > targetmanamax) then
		targetmanamax = targetmana;
	end

	frameManabar:SetMinMaxValues(0, targetmanamax);
	frameManabar:SetValue(targetmana);

	frameManaBarPercent:SetText(string.format("%d",(100*(targetmana / targetmanamax))+0.5).."%");
	frameManaBarText:SetText(targetmana.."/"..targetmanamax);
end

local function Perl_Target_UpdateHealth()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetHealth("target", Perl_Target_HealthBar, Perl_Target_HealthBarBG, Perl_Target_HealthBarPercent, Perl_Target_HealthBarText, Perl_Target_ManaBarPercent)

	if (not UnitIsConnected("target")) then
		Perl_Target_Frame:SetAlpha(Perl_Config.Transparency / 2);
	else
		Perl_Target_Frame:SetAlpha(Perl_Config.Transparency);
	end
end
function Perl_Target_SetHealth(unit, frameHealthBar, frameHealthBarBG, frameHealthBarPercent, frameHealthBarText, frameManaBarPercent)
	local targethealth = UnitHealth(unit);
	local targethealthmax = UnitHealthMax(unit);

	frameHealthBar:SetMinMaxValues(0, targethealthmax);
	frameHealthBar:SetValue(targethealth);
	Perl_SetSmoothBarColor(frameHealthBar);
	Perl_SetSmoothBarColor(frameHealthBarBG, frameHealthBar, 0.25);
	if (targethealthmax == 100) then
		frameHealthBarPercent:SetText(targethealth.."%");
	else
		frameHealthBarPercent:SetText(string.format("%d",(100*(targethealth / targethealthmax))+0.5).."%");
	end
	if (targethealthmax ~= 100) then
		frameHealthBarText:SetText(targethealth.."/"..targethealthmax);
	else
		-- Hide MobInfo-2 frame
		if MobHealthFrame then
			MobHealthFrame:Hide();
		end

		local index;
		if UnitIsPlayer(unit) then
			index = UnitName(unit);
		else
			index = UnitName(unit)..":"..UnitLevel(unit);
		end

		local table = MobHealthDB;
		if (not (table and table[index])) then
			table = MobHealthPlayerDB;
		end
		if (table and table[index]) then
			local s, e;
			local pts;
			local pct;

			if ( type(table[index]) ~= "string" ) then
				frameHealthBarText:SetText(targethealth.."%");
			end
			s, e, pts, pct = string.find(table[index], "^(%d+)/(%d+)$");

			if ( pts and pct ) then
				pts = pts + 0;
				pct = pct + 0;
				if( pct ~= 0 ) then
					pointsPerPct = pts / pct;
				else
					pointsPerPct = 0;
				end
			end
			local currentPct = UnitHealth(unit);
			if ( pointsPerPct > 0 ) then
				frameHealthBarText:SetText(string.format("%d", (currentPct * pointsPerPct) + 0.5).."/"..string.format("%d", (100 * pointsPerPct) + 0.5));
			end
		else
			frameHealthBarText:SetText(targethealth.."%");
		end
	end

	if (UnitIsGhost(unit)) then
		frameManaBarPercent:Hide();
		frameHealthBarPercent:SetText("Ghost");
	elseif (UnitIsDead(unit)) then
		frameManaBarPercent:Hide();
		frameHealthBarPercent:SetText("Dead");
	elseif (UnitName(unit) and not UnitIsConnected(unit)) then
		frameManaBarPercent:Hide();
		frameHealthBarPercent:SetText("Offline");
	end
end

function Perl_Target_UpdateDisplay()
	if (perl_loaded) then
		if (UnitName("target") ~= nil) then
			this.lastTarget = UnitName("target");
			this.lastDeathMsg = "";
			Perl_CancelFade(Perl_Target_Frame);
			--Perl_CancelFade(Perl_Target_Assists_Frame);

			Perl_Target_Frame:Show();
			Perl_Target_UpdateName();
			--Perl_Target_UpdatePortrait();
			Perl_Target_UpdateLevel();
			Perl_Target_UpdateClassification();
			Perl_Target_UpdateType();
			Perl_Target_UpdatePVP();
			Perl_Target_UpdateCombo();
			Perl_Target_UpdateManaType();
			Perl_Target_UpdateMana();
			Perl_Target_UpdateHealth();
                        Perl_Target_Update_Combat();
			Perl_Target_Buff_UpdateAll();
			Perl_Target_DebuffUpdate();

			if (not UnitIsConnected("target")) then
				Perl_Target_Frame:SetAlpha(Perl_Config.Transparency / 2);
			else
				Perl_Target_Frame:SetAlpha(Perl_Config.Transparency);
			end

			TargetFrame:Hide();  -- Hide default frame
			ComboFrame:Hide();  -- Hide Combo Points
		else
			if (this.lastDeathMsg ~= nil and this.lastTarget ~= nil) then
				if (string.find (this.lastDeathMsg, this.lastTarget)) then
					Perl_Target_UpdateCombo();
					this.lastDeathMsg = "";
					Perl_Target_HealthBar:SetMinMaxValues(0, 1);
					Perl_Target_HealthBar:SetValue(0);
					Perl_Target_HealthBarBG:SetStatusBarColor(1, 0, 0, 0.25);
					Perl_Target_ManaBar:SetMinMaxValues(0, 1);
					Perl_Target_ManaBar:SetValue(0);

					local oldhp = Perl_Target_HealthBarText:GetText();
					if (string.find(oldhp, "/")) then
						oldhp = string.gsub(oldhp, "%d+/(%d+)", "0/%1");
						Perl_Target_HealthBarText:SetText(oldhp);
					else
						Perl_Target_HealthBarText:SetText("0%");
					end

					local oldmana = Perl_Target_ManaBarText:GetText();
					if (string.find(oldhp, "/")) then
						oldmana = string.gsub(oldmana, "%d+/(%d+)", "0/%1");
						Perl_Target_ManaBarText:SetText(oldmana);
					else
						Perl_Target_ManaBarText:SetText("0%");
					end

					Perl_Target_HealthBarPercent:SetText("Dead");
				end
			end

			Perl_StartFade(Perl_Target_Frame);
			--Perl_StartFade(Perl_Target_Assists_Frame);
			if (Perl_Target_HealthBarPercent:GetText() == "Offline") then
				Perl_Target_Frame.FadeTime = 0.5;
			end
		end
	end
end

--------------------
-- Click Handlers --
--------------------

function Perl_TargetDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_TargetDropDown_Initialize, "MENU");
end

function Perl_TargetDropDown_Initialize()
	local menu = nil;
	--if ( UnitIsEnemy("target", "player") ) then
		menu = "RAID_TARGET_ICON";		-- 1.11 Target icon menu
	--end
	if ( UnitIsUnit("target", "player") ) then
		menu = "SELF";
	elseif ( UnitIsUnit("target", "pet") ) then
		menu = "PET";
	elseif ( UnitIsPlayer("target") and (UnitFactionGroup("player") == UnitFactionGroup("target"))) then
		if ( UnitInParty("target") ) then
			menu = "PARTY";
		else
			menu = "PLAYER";
		end
	end
	if ( menu ) then
		UnitPopup_ShowMenu(Perl_Target_DropDown, menu, "target");
	end
end

function Perl_Target_MouseUp(button)
	-- If player clicks the target frame while it's fading away. no actual target, but we can get that back:
	if (not UnitName("target")) then
		TargetLastTarget();
	end

	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end
	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			SpellTargetUnit("target");
		elseif ( CursorHasItem() ) then
			DropItemOnUnit("target");
		end
	else
		ToggleDropDownMenu(1, nil, Perl_Target_DropDown, "Perl_Target_NameFrame", 40, -25);
	end

	Perl_Target_Frame:StopMovingOrSizing();
end

local function SetFrameSides()
	if (Perl_Target_Assists_Frame.LastSetView and Perl_Target_Assists_Frame.LastSetView[1] == Perl_Config.AssistsFrame and Perl_Target_Assists_Frame.LastSetView[2] == Perl_Config.TargettingFrame) then
		-- Frames the same from last time
		return;
	end

	if (Perl_Config.AssistsFrame == 1 or Perl_Config.TargettingFrame == 1) then
        	Perl_Target_Assists_Frame:Show();

		Perl_Target_Targetting_ScrollFrame:ClearAllPoints();
		Perl_Target_Assists_ScrollFrame:ClearAllPoints();

		if (Perl_Config.AssistsFrame == 1 and Perl_Config.TargettingFrame == 1) then
			Perl_Target_Targetting_ScrollFrame:SetPoint("TOPLEFT", 4, -4);
			Perl_Target_Targetting_ScrollFrame:SetPoint("BOTTOMRIGHT", Perl_Target_Assists_Frame, "BOTTOM", -1, 4);
			Perl_Target_Targetting_ScrollFrame:Show();

			Perl_Target_Assists_ScrollFrame:SetPoint("TOPLEFT", Perl_Target_Assists_Frame, "TOP", 0, -4);
			Perl_Target_Assists_ScrollFrame:SetPoint("BOTTOMRIGHT", -4, 4);
			Perl_Target_Assists_ScrollFrame:Show();

			PerlScrollSeperator:Show();
			PerlScrollSeperator:ClearAllPoints();
			PerlScrollSeperator:SetPoint("TOPLEFT", Perl_Target_Targetting_ScrollFrame, "TOPRIGHT", 0, 0);
			PerlScrollSeperator:SetPoint("BOTTOMRIGHT", Perl_Target_Assists_ScrollFrame, "BOTTOMLEFT", 0, 0);
		else
			PerlScrollSeperator:Hide();

			if (Perl_Config.AssistsFrame == 1) then
				Perl_Target_Assists_ScrollFrame:SetPoint("TOPLEFT", 4, -4);
				Perl_Target_Assists_ScrollFrame:SetPoint("BOTTOMRIGHT", -4, 4);
				Perl_Target_Assists_ScrollFrame:Show();
				Perl_Target_Targetting_ScrollFrame:Hide();
			else
				Perl_Target_Targetting_ScrollFrame:SetPoint("TOPLEFT", 4, -4);
				Perl_Target_Targetting_ScrollFrame:SetPoint("BOTTOMRIGHT", -4, 4);
				Perl_Target_Targetting_ScrollFrame:Show();
				Perl_Target_Assists_ScrollFrame:Hide();
			end
		end
	else
        	Perl_Target_Assists_Frame:Hide();
	end

	Perl_Target_Assists_Frame.LastSetView = {Perl_Config.AssistsFrame, Perl_Config.TargettingFrame};
end

-- ToggleAssistsFrame()
function ToggleAssistsFrame(param)
	if (param == "assists") then
		if (Perl_Config.AssistsFrame == 1) then
			Perl_Config.AssistsFrame = 0;
		else
			Perl_Config.AssistsFrame = 1;
		end
	else
		if (Perl_Config.TargettingFrame == 1) then
			Perl_Config.TargettingFrame = 0;
		else
			Perl_Config.TargettingFrame = 1;
		end
	end
end

-- Perl_AssistsView_Close
function Perl_AssistsView_Close()
	Perl_Config.AssistsFrame = 0;
	Perl_Config.TargettingFrame = 0;
	SetFrameSides();
end

-- SortByClass(t1, t2)
local function SortByClass(t1, t2)
	if (t1[2] == t2[2]) then
		return t1[1] < t2[1];
	else
		local me = UnitClass("player");
		local t1c = t1[2];
		local t2c = t2[2];
		if (t1c == me) then
			t1c = "A"..t1c;
		elseif (t1c ~= "") then
			t1c = "B"..t1c;
		else
			t1c = "Z";
		end
		if (t2c == me) then
			t2c = "A"..t2c;
		elseif (t2c ~= "") then
			t2c = "B"..t2c;
		else
			t1c = "Z";
		end

		return t1c..t1[1] < t2c..t2[1];
	end
end

local function FillList(List, cFrame, title)
	local lastClass;
	local any = false;
	local didEnemyBreak = false;

	local text = title;
	if (List ~= nil) then
		local nAssists = table.getn (List);
		if (nAssists > 0) then
			text = text.." "..nAssists;
		end
		text = text.."\13";

		getglobal(cFrame.."_List1_Text"):SetText(text);

		table.sort(List, SortByClass);

		local row = 2;
		text = "";
		for i = 1,nAssists do
			local unit = List[i];

			if (not any) then
				if (unit[2] == "") then
					text = "|c00FF0000"..unit[1];
				else
					text = PerlColourTable[unit[2]]..unit[1];
				end
				lastClass = unit[2];
				any = true;
			else
				if (lastClass) then
					if (unit[2] == "" or lastClass ~= unit[2]) then
						lastClass = unit[2];
						getglobal(cFrame.."_List"..row.."_Text"):SetText(text);
						text = "";
						row = row + 1;

						if (unit[2] == "") then
							text = "|c00FF0000"..unit[1];
						else
							text = PerlColourTable[unit[2]]..unit[1];
						end
					else
						text = text.." "..unit[1];
					end
				else
					text = text.." "..unit[1];

					lastClass = unit[2];
				end
			end
		end

		for i = row,12 do
			row = i;
			getglobal(cFrame.."_List"..row.."_Text"):SetText(text);
			text = "";
		end
	end
end

-- Perl_Target_ShowAssists()
function Perl_Target_ShowAssists()
	SetFrameSides();

	if (Perl_Config.AssistsFrame == 1 or Perl_Config.TargettingFrame == 1) then
		if (Perl_Config.AssistsFrame == 1 and Perl_Target_AssistFrame.assists ~= nil) then
			FillList(Perl_Target_AssistFrame.assists, "Perl_Target_Assists_ScrollChild_Assists", PERL_TOOLTIP_ASSISTING);
		end

		if (Perl_Config.TargettingFrame == 1 and Perl_Target_AssistFrame.targetting ~= nil) then
			local title;
			if (table.getn(Perl_Target_AssistFrame.targetting) > 0 and Perl_Target_AssistFrame.targetting[1][2] == "") then
				title = PERL_TOOLTIP_ENEMYONME;
			else
				if (Perl_Config.TargetCountersSelf == 0) then
					title = PERL_TOOLTIP_ALLONME;
				else
					title = PERL_TOOLTIP_HEALERS;
				end
			end

			FillList(Perl_Target_AssistFrame.targetting, "Perl_Target_Assists_ScrollChild_Targetting", title);
		end
	end

end

function Perl_Target_Assists_MouseDown(button,param)
	if (button == "LeftButton") then
		if (param == nil) then
			Perl_Target_Assists_Frame:StartMoving();
		else
			Perl_Target_Assists_Frame:StartSizing(param);
		end
	elseif (button == "RightButton") then
		if (string.find (this:GetName(), "Perl_Target_Assists_ScrollChild_Targetting")) then
			param = "targetFrame";
		end

		if (param and param == "targetFrame") then
			if (Perl_Config.TargetCountersSelf == 1) then
				Perl_Config.TargetCountersSelf = 0;
			else
				Perl_Config.TargetCountersSelf = 1;
			end
		end
	end
end

function Perl_Target_Assists_MouseUp(button)
	if (button == "LeftButton") then
		Perl_Target_Assists_Frame:StopMovingOrSizing();
	end
end

function Perl_Target_MouseDown(button, param)
	if (param == "assists") then
		ToggleAssistsFrame("assists");
	else
		if ( button == "LeftButton" and perl_locked == 0) then
			if (this:GetName()=="Perl_Target_ComboBubbles_Overlay") then
				Perl_Target_ComboBubbles:StartMoving();
			else
				Perl_Target_Frame:StartMoving();
			end
		end
	end
end

local function Perl_Target_Update_Control()
        if (UnitIsVisible("target") and UnitIsCharmed("target")) then
		Perl_Target_Warning:Show();
	else
		Perl_Target_Warning:Hide();
	end
end

function Perl_Target_Update_Combat()
        if (UnitAffectingCombat("target")) then
                Perl_Target_ActivityStatus:SetTexCoord(0.5, 1.0, 0.0, 0.5);
                Perl_Target_ActivityStatus:Show();
        else
                Perl_Target_ActivityStatus:Hide();
        end
	Perl_Target_Update_Control();
end

local function Perl_Target_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Target_PortraitFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_BossFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_TypeFramePlayer, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_AssistFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_TypeFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function Perl_Target_Udpate_Range()

	if (CheckInteractDistance("target", 4)) then
		Perl_Target_RangeStatus:Hide();
	else
		Perl_Target_RangeStatus:SetTexCoord(0.5, 0.75, 1.0, 0.0);
		Perl_Target_RangeStatus:SetAlpha(0.6);
		Perl_Target_RangeStatus:Show();
	end
end

function Perl_Target_OnUpdate()
	if (UnitName("target")) then
	        Perl_Target_Update_Combat();
		Perl_Target_Udpate_Range();
		Perl_Player_UpdateAssists();
		Perl_Target_ShowAssists();
		Perl_Target_CombatFlash(arg1, false);
	end

	Perl_ProcessFade(Perl_Target_Frame);
end

-------------------
-- Event Handler --
-------------------
function Perl_Target_OnEvent()
	local func = Perl_Target_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Target_Events:PLAYER_ENTERING_WORLD()
	Perl_TargetTarget_ClearHistory();
	Perl_Target_RegisterSome();
end

-- VARIABLES_LOADED
function Perl_Target_Events:VARIABLES_LOADED()
	Perl_Target_RegisterSome();
end

-- PLAYER_LEAVING_WORLD
function Perl_Target_Events:PLAYER_LEAVING_WORLD()
	Perl_Target_Frame.Fading = 0;
	Perl_Target_Frame.lastTarget = 0;
	Perl_Target_Frame:Hide();
	Perl_Target_Assists_Frame:Hide();
	Perl_Target_UnregisterSome();
end

-- UNIT_COMBAT
function Perl_Target_Events:UNIT_COMBAT()
	if (arg1 == "target") then
	        Perl_Target_Update_Combat();

		CombatFeedback_OnCombatEvent(arg2, arg3, arg4, arg5);

		if (arg2 == "HEAL") then
			Perl_Target_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Target_CombatFlash(0, true);
		end
	end
end

-- UNIT_SPELLMISS
function Perl_Target_Events:UNIT_SPELLMISS()
	if (arg1 == "target") then
        	Perl_Target_Update_Combat();
		CombatFeedback_OnSpellMissEvent(arg2);
	end
end

-- PLAYER_TARGET_CHANGED
function Perl_Target_Events:PLAYER_TARGET_CHANGED()
	Perl_TargetTarget_ClearHistory();
	Perl_Target_Frame.PlayerFlash = 0;
	Perl_Target_CombatFlash(0, false);
	Perl_Target_UpdateDisplay();
	Perl_Player_UpdateAssists();
	Perl_Target_ShowAssists();
	Perl_Target_UpdatePortrait();
	return;
end

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Target_Events:UNIT_HEALTH()
	if (arg1 == "target") then
		Perl_Target_UpdateHealth();
	end
end
Perl_Target_Events.UNIT_MAXHEALTH = Perl_Target_Events.UNIT_HEALTH;

-- UNIT_DYNAMIC_FLAGS
function Perl_Target_Events:UNIT_DYNAMIC_FLAGS()
	if (arg1 == "target") then
		Perl_Target_UpdateName();
		--Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
	end
end

Perl_Target_Events.UNIT_FLAGS = Perl_Target_Events.UNIT_DYNAMIC_FLAGS;

-- UNIT_MANA, UNIT_MAXMANA, UNIT_RAGE, UNIT_MAXRAGE, UNIT_ENERGY
-- UNIT_MAXENERGY, UNIT_FOCUS, UNIT_MAXFOCUS
function Perl_Target_Events:UNIT_MANA()
	if (arg1 == "target") then
		Perl_Target_UpdateMana();
	end
end
Perl_Target_Events.UNIT_MAXMANA		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_RAGE		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXRAGE		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_ENERGY		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXENERGY	= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_FOCUS		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXFOCUS	= Perl_Target_Events.UNIT_MANA;

-- UNIT_DISPLAYPOWER
function Perl_Target_Events:UNIT_DISPLAYPOWER()
	if (arg1 == "target") then
		Perl_Target_UpdateManaType();
		Perl_Target_UpdateMana();
	end
end

-- UNIT_MODEL_CHANGED
function Perl_Target_Events:UNIT_MODEL_CHANGED()
	if (arg1 == "target") then
		Perl_Target_UpdatePortrait();
	end
end

-- UNIT_NAME_UPDATE
function Perl_Target_Events:UNIT_NAME_UPDATE()
	if (arg1 == "target") then
		Perl_Target_UpdateName();
	end
end

-- UNIT_LEVEL
function Perl_Target_Events:UNIT_LEVEL()
	if (arg1 == "target") then
		Perl_Target_UpdateLevel()
	end
end

-- UNIT_CLASSIFICATION_CHANGED
function Perl_Target_Events:UNIT_CLASSIFICATION_CHANGED()
	if (arg1 and arg1 == "target") then
		Perl_Target_UpdateClassification();
	end
end

-- PLAYER_COMBO_POINTS
function Perl_Target_Events:PLAYER_COMBO_POINTS()
	Perl_Target_UpdateCombo();
end

-- UNIT_AURA
function Perl_Target_Events:UNIT_AURA()
	if (arg1 == "target") then
		Perl_Target_Buff_UpdateAll();
		Perl_Target_DebuffUpdate();
	end
end

-- UNIT_PVP_UPDATE
function Perl_Target_Events:UNIT_PVP_UPDATE()
	if (arg1 == "target") then
		Perl_Target_UpdatePVP();
		Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
	end
end

Perl_Target_Events.UNIT_FACTION = Perl_Target_Events.UNIT_PVP_UPDATE;

-- PLAYER_FLAGS_CHANGED
function Perl_Target_Events:PLAYER_FLAGS_CHANGED()
	Perl_Target_Update_Combat();
	Perl_Target_UpdatePVP();
	Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
	Perl_Target_UpdateHealth();
end

-- CHAT_MSG_COMBAT_HOSTILE_DEATH
function Perl_Target_Events:CHAT_MSG_COMBAT_HOSTILE_DEATH()
	if (string.find(arg1, this.lastTarget)) then
		this.lastDeathMsg = arg1;
	end
end

function Perl_Target_Events:PARTY_MEMBER_ENABLE()
	if (UnitInParty("target") or UnitInRaid("target")) then
		Perl_Target_Update_Combat();
		Perl_Target_UpdatePVP();
		Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
		Perl_Target_UpdateHealth();
	end
end

Perl_Target_Events.PARTY_MEMBER_DISABLE = Perl_Target_Events.PARTY_MEMBER_ENABLE;
